/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.cli.extension.command.monitoring.mo.components;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.object.ExhaustableObjectPoolMetrics;

/**
 * An abstract monitoring sub-function managing exhaustable object pool.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractExhaustableObjectPool extends AbstractComponentMonitoringSubFunction implements
        MonitoringSubFunction {

    /**
     * Header of the metric value associated to the max borrowed objects of the
     * object pool
     */
    public static final String HEADER_BORROWED_OBJECTS_MAX = "BorrowedObjectsMax";

    /**
     * Header of the metric value associated to the current borrowed objects of
     * the object pool
     */
    public static final String HEADER_BORROWED_OBJECTS_CURRENT = "BorrowedObjectsCurrent";

    /**
     * Header of the metric value associated to the idle objects max number of
     * the object pool
     */
    public static final String HEADER_IDLE_OBJECTS_MAX = "IdleObjectsMax";

    /**
     * Header of the metric value associated to the idle object current objects
     * of the thread pool
     */
    public static final String HEADER_IDLE_OBJECTS_CURRENT = "IdleObjectsCurrent";

    /**
     * Header of the metric value associated to the max size of the object pool
     */
    public static final String HEADER_MAX_SIZE = "MaxSize";

    /**
     * Header of the metric value associated to the min idle size of the object
     * pool
     */
    public static final String HEADER_MIN_IDLE_SIZE = "MinIdleSize";

    /**
     * Header of the metric value associated to the number of exhaustion of the
     * object pool
     */
    public static final String HEADER_EXHAUSTIONS = "Exhaustions";

    /**
     * Get the current metrics of the exhaustable object pool of the component
     * 
     * @param client
     *            The JMX client to use to retrieve current metrics of the object pool
     * @param componentIdentifier
     *            Identifier of the component
     * @return The current metrics of the exhaustable object pool
     * @throws ComponentMonitoringServiceDoesNotExistException
     *             The monitoring service needed to retrieve current metrics of the object pool does not exist.
     * @throws ComponentMonitoringServiceErrorException
     *             An error occurs retrieving the current metrics of the object pool.
     * @throws ConnectionErrorException
     *             A connection error occurs
     * 
     */
    protected abstract ExhaustableObjectPoolMetrics getExhaustableObjectPoolMetrics(final JMXClient client,
            final String componentIdentifier)
            throws ComponentMonitoringServiceDoesNotExistException,
            ComponentMonitoringServiceErrorException, ConnectionErrorException;

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final JMXClient client = getJMXClient();
            try {

                final ExhaustableObjectPoolMetrics objectPoolMetrics = this.getExhaustableObjectPoolMetrics(client,
                        componentIdentifier);

                final StringBuilder reply = new StringBuilder();
                final Long borrowedObjectsMax = objectPoolMetrics.getBorrowedObjectMaxNumber();
                if (borrowedObjectsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_BORROWED_OBJECTS_MAX,
                            borrowedObjectsMax.longValue()));
                }

                final Long borrowedObjectsCurrent = objectPoolMetrics.getBorrowedObjectNumber();
                if (borrowedObjectsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_BORROWED_OBJECTS_CURRENT,
                            borrowedObjectsCurrent.longValue()));
                }

                final Long idleObjectsMax = objectPoolMetrics.getIdleObjectMaxNumber();
                if (idleObjectsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_IDLE_OBJECTS_MAX,
                            idleObjectsMax.longValue()));
                }

                final Long idleObjectsCurrent = objectPoolMetrics.getIdleObjectNumber();
                if (idleObjectsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_IDLE_OBJECTS_CURRENT,
                            idleObjectsCurrent.longValue()));
                }

                final Long maxSize = objectPoolMetrics.getMaxSize();
                if (maxSize != null) {
                    reply.append(String.format("%s:%d ", HEADER_MAX_SIZE, maxSize.longValue()));
                }

                final Long minIdleSize = objectPoolMetrics.getMinIdleSize();
                if (minIdleSize != null) {
                    reply.append(String.format("%s:%d ", HEADER_MIN_IDLE_SIZE,
                            minIdleSize.longValue()));
                }

                final Long exhaustions = objectPoolMetrics.getExhaustions();
                if (exhaustions != null) {
                    reply.append(String.format("%s:%d ", HEADER_EXHAUSTIONS,
                            exhaustions.longValue()));
                }

                this.shell.getPrintStream().println(reply.toString());

            } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
