/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;

/**
 * The monitoring sub-function about the message exchange acceptors of a component.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class MexAcceptors extends AbstractComponentMonitoringSubFunction implements MonitoringSubFunction {

    /**
     * The monitoring sub-function name
     */
    private final static String SUBFCT_NAME = "mex-acceptor-working";

    /**
     * Monitoring sub-function description
     */
    private final static String SUBFCT_DESCRIPTION = "Statistics about working message exchange acceptors";

    /**
     * Header of the metric value associated to the max number of working message exchange acceptors.
     */
    public static final String HEADER_ACCEPTOR_WORKING_NB_MAX = "MexAcceptorWorkingNbMax";

    /**
     * Header of the metric value associated to the current number of working message exchange acceptors.
     */
    public static final String HEADER_ACCEPTOR_WORKING_NB_CURRENT = "MexAcceptorWorkingNbCurrent";

    /**
     * Header of the metric value associated to the max duration on time sample of message exchange acceptor processing.
     */
    public static final String HEADER_ACCEPTOR_DURATION_MAX = "MexAcceptorWorkingDurationMax";

    /**
     * Header of the metric value associated to the avg duration on time sample of message exchange acceptor processing.
     */
    public static final String HEADER_ACCEPTOR_DURATION_AVG = "MexAcceptorWorkingDurationAvg";

    /**
     * Header of the metric value associated to the min duration on time sample of message exchange acceptor processing.
     */
    public static final String HEADER_ACCEPTOR_DURATION_MIN = "MexAcceptorWorkingDurationMin";

    /**
     * Header of the metric value associated to the 10-percentile duration on time sample of message exchange acceptor
     * processing.
     */
    public static final String HEADER_ACCEPTOR_DURATION_10P = "MexAcceptorWorkingDuration10p";

    /**
     * Header of the metric value associated to the 50-percentile duration on time sample of message exchange acceptor
     * processing.
     */
    public static final String HEADER_ACCEPTOR_DURATION_50P = "MexAcceptorWorkingDuration50p";

    /**
     * Header of the metric value associated to the 10-percentile duration on time sample of message exchange acceptor
     * processing.
     */
    public static final String HEADER_ACCEPTOR_DURATION_90P = "MexAcceptorWorkingDuration90p";

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final JMXClient client = getJMXClient();
            try {

                final ComponentMonitoringServiceClient monitoringService = client
                        .getComponentMonitoringServiceClient(componentIdentifier);

                final StringBuilder reply = new StringBuilder();
                final Long mexAcceptorWorkingMax = monitoringService
                        .getMexAcceptorWorkingMax();
                if (mexAcceptorWorkingMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_ACCEPTOR_WORKING_NB_MAX,
                            mexAcceptorWorkingMax.longValue()));
                }

                final Long mexAcceptorWorkingCurrent = monitoringService
                        .getMexAcceptorWorkingCurrent();
                if (mexAcceptorWorkingCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_ACCEPTOR_WORKING_NB_CURRENT,
                            mexAcceptorWorkingCurrent.longValue()));
                }
                
                final Long[] mexAcceptorDurations = monitoringService.getMexAcceptorRelativeDurations();
                if (mexAcceptorDurations != null) {
                    reply.append(String.format("%s:%d %s:%d %s:%d %s:%d %s:%d %s:%d", HEADER_ACCEPTOR_DURATION_MAX,
                            mexAcceptorDurations[0], HEADER_ACCEPTOR_DURATION_AVG, mexAcceptorDurations[1],
                            HEADER_ACCEPTOR_DURATION_MIN, mexAcceptorDurations[2], HEADER_ACCEPTOR_DURATION_10P,
                            mexAcceptorDurations[3], HEADER_ACCEPTOR_DURATION_50P, mexAcceptorDurations[4],
                            HEADER_ACCEPTOR_DURATION_90P,
                            mexAcceptorDurations[5]));
                }

                this.shell.getPrintStream().println(reply.toString());

            } catch (final ComponentMonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this, String.format(
                        Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
