/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_D;
import static org.ow2.petals.cli.extension.command.monitoring.mo.Constants.QUERY_RESULT_OUTPUT_FORMAT_S_S;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationKey;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ServiceProviderInvocationStatus;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;

/**
 * The monitoring sub-function about service provider invocation counters of the component.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ServiceProviderInvocationsCount extends AbstractComponentMonitoringSubFunction {

    private final static String SUBFCT_NAME = "service-provider-invocations-count";

    private final static String SUBFCT_DESCRIPTION = "Counters of service provider requests";

    /**
     * The option to get the list of external web service operations
     */
    public static final String LIST_OPERATIONS_LONG_OPTION = "list-operations";

    private static final Option LIST_OPERATIONS_OPTION = Option.builder().required(false).hasArg(false)
            .desc("List the operations for which at least one request was sent.").longOpt(LIST_OPERATIONS_LONG_OPTION)
            .build();

    /**
     * The option to query outgoing requests of external web-service operations
     */
    public static final String QUERY_OPERATIONS_LONG_OPTION = "query-operations";

    private static final Option QUERY_OPERATIONS_OPTION = Option.builder().required(false).optionalArg(true)
            .numberOfArgs(2).desc("Query about operations").longOpt(QUERY_OPERATIONS_LONG_OPTION).build();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = Option.builder(TABLE_SHORT_OPTION).required(false)
            .desc("Get the complete table of values").build();

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the number of outgoing ws-requests successfuly
     * processed.
     */
    public static final String QUERY_ATTR_SUCCEEDED = "SUCCEEDED";

    /**
     * Query attribute to get the number of outgoing pending ws-requests.
     */
    public static final String QUERY_ATTR_PENDING = "PENDING";

    /**
     * Query attribute to get the number of outgoing ws-requests faulty
     * processed.
     */
    public static final String QUERY_ATTR_FAULT = "FAULT";

    /**
     * Query attribute to get the number of outgoing ws-requests for which an
     * error occurs
     */
    public static final String QUERY_ATTR_ERROR = "ERROR";

    private final static String HEADER_INTERFACE_NAME = "Interface";

    private final static String HEADER_SERVICE_NAME = "Service";

    private final static String HEADER_OPERATION = "Operation";

    private final static String HEADER_MEP = "MEP";

    private final static String HEADER_EXEC_STATUS = "Execution status";

    private final static String HEADER_VALUE = "Value";

    // TODO: Options declaration must be reviewed to match:
    // -n <component-name>
    // [--list-operations] |
    // [--query-operations [NAME] | [SUCCEEDED|FAULT|ERROR <index>]] |
    // [-t]
    @Override
    protected void createOptions() {
        super.createOptions();
        getOptions().addOption(LIST_OPERATIONS_OPTION);
        getOptions().addOption(QUERY_OPERATIONS_OPTION);
        getOptions().addOption(TABLE_OPTION);
    }

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            // No extra option is accepted
            if (cli.getArgs().length > 0) {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            if (cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listOperations(componentIdentifier, false, null);

            } else if (!cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                final String[] queryArgs = cli.getOptionValues(QUERY_OPERATIONS_LONG_OPTION);
                if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null
                        || queryArgs[0].isEmpty()) {
                    throw new MonitoringSubFunctionMissingArgumentException(this,
                            QUERY_OPERATIONS_OPTION);
                } else if (!QUERY_ATTR_NAME.equals(queryArgs[0])
                        && !QUERY_ATTR_SUCCEEDED.equals(queryArgs[0])
                        && !QUERY_ATTR_PENDING.equals(queryArgs[0])
                        && !QUERY_ATTR_FAULT.equals(queryArgs[0])
                        && !QUERY_ATTR_ERROR.equals(queryArgs[0])) {
                    throw new MonitoringSubFunctionInvalidArgumentException(this,
                            QUERY_OPERATIONS_OPTION, queryArgs[0]);
                } else {
                    this.listOperations(componentIdentifier, true,
                            cli.getOptionValues(QUERY_OPERATIONS_LONG_OPTION));
                }

            } else if (!cli.hasOption(LIST_OPERATIONS_LONG_OPTION)
                    && !cli.hasOption(QUERY_OPERATIONS_LONG_OPTION)
                    && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable(componentIdentifier);

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    /**
     * List operations for which a metric about service provider invocations is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listOperations(final String componentIdentifier, final boolean hasQuery,
            final String[] queryArgs) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<ServiceProviderInvocationKey, Long> values = client
                    .getComponentMonitoringServiceClient(componentIdentifier)
                    .getServiceProviderInvocationsCount();
            if (values != null) {
                if (hasQuery) {
                    this.query(values, queryArgs, QUERY_OPERATIONS_OPTION);
                } else {
                    final List<String> operations = new ArrayList<>();
                    for (final ServiceProviderInvocationKey key : values.keySet()) {
                        final String operation = ServiceProviderInvocationsCount.getOperationName(key);
                        if (!operations.contains(operation)) {
                            operations.add(operation);
                        }
                    }

                    for (final String operation : operations) {
                        this.shell.getPrintStream().println(operation);
                    }
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, String.format(
                    Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
        }
    }

    private static String getOperationName(final ServiceProviderInvocationKey key) {
        return key.getOperation() + "@" + key.getServiceName() + "@" + key.getInterfaceName();
    }

    private void query(final Map<ServiceProviderInvocationKey, Long> values,
            final String[] queryArgs,
            final Option queryOption) throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null
                || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            final String index = queryArgs.length == 2 ? queryArgs[1] : null;
            final Map<String, Long> valuesByStatus;
            if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                final List<String> operations = new ArrayList<>();
                for (final ServiceProviderInvocationKey key : values.keySet()) {
                    final String operation = ServiceProviderInvocationsCount.getOperationName(key);
                    if (!operations.contains(operation)) {
                        operations.add(operation);
                    }
                }

                for (final String operation : operations) {
                    this.shell.getPrintStream().println(
                            String.format(QUERY_RESULT_OUTPUT_FORMAT_S_S, operation, operation));
                }
            } else {
                if (QUERY_ATTR_SUCCEEDED.equals(queryArgs[0])) {
                    valuesByStatus = this.agregateByExecStatus(values,
                            ServiceProviderInvocationStatus.SUCCEEDED);
                } else if (QUERY_ATTR_PENDING.equals(queryArgs[0])) {
                    valuesByStatus = this.agregateByExecStatus(values,
                            ServiceProviderInvocationStatus.PENDING);
                } else if (QUERY_ATTR_FAULT.equals(queryArgs[0])) {
                    valuesByStatus = this.agregateByExecStatus(values,
                            ServiceProviderInvocationStatus.FAULT);
                } else if (QUERY_ATTR_ERROR.equals(queryArgs[0])) {
                    valuesByStatus = this.agregateByExecStatus(values,
                            ServiceProviderInvocationStatus.ERROR);
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption,
                            queryArgs[0]);
                }

                if (index != null) {
                    final Long value = valuesByStatus.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%s", value));
                    }
                } else {
                    for (final Map.Entry<String, Long> entry : valuesByStatus.entrySet()) {
                        this.shell.getPrintStream().println(
                                String.format(QUERY_RESULT_OUTPUT_FORMAT_S_D, entry.getKey(),
                                        entry.getValue()));
                    }
                }
            }
        }
    }

    private Map<String, Long> agregateByExecStatus(
final Map<ServiceProviderInvocationKey, Long> values,
            final ServiceProviderInvocationStatus status) {
        final Map<String, Long> results = new HashMap<>();
        for (final Map.Entry<ServiceProviderInvocationKey, Long> entry : values.entrySet()) {
            final ServiceProviderInvocationKey key = entry.getKey();
            if (key.getExecStatus() == status) {
                final String operation = ServiceProviderInvocationsCount.getOperationName(key);
                final Long agregatedValue = results.get(operation);
                results.put(operation, agregatedValue == null ? entry.getValue() : agregatedValue
                        + entry.getValue());
            }
        }
        return results;
    }

    /**
     * Print the full table
     * 
     * @param componentIdentifier
     *            The identifier of the component
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable(final String componentIdentifier) throws MonitoringSubFunctionException {

        try {
            final JMXClient client = getJMXClient();
            final Map<ServiceProviderInvocationKey, Long> values = client
                    .getComponentMonitoringServiceClient(componentIdentifier)
                    .getServiceProviderInvocationsCount();

            int maxSizeItf = 0;
            int maxSizeSvc = 0;
            int maxSizeOp = 0;
            int maxSizeMep = 0;
            int maxSizeExecStatus = 0;
            int maxSizeValue = 0;
            if (values != null) {
                for (final Map.Entry<ServiceProviderInvocationKey, Long> entry : values.entrySet()) {
                    final ServiceProviderInvocationKey key = entry.getKey();
                    final String interfaceName = key.getInterfaceName();
                    maxSizeItf = Math.max(maxSizeItf,
                            interfaceName == null ? 0 : interfaceName.length());
                    final String serviceName = key.getServiceName();
                    maxSizeSvc = Math.max(maxSizeSvc,
                            serviceName == null ? 0 : serviceName.length());
                    final String operation = key.getOperation();
                    maxSizeOp = Math.max(maxSizeOp, operation == null ? 0 : operation.length());
                    final String mep = key.getMep();
                    maxSizeMep = Math.max(maxSizeMep, mep == null ? 0 : mep.length());
                    final String execStatus = key.getExecStatus().toString();
                    maxSizeExecStatus = Math.max(maxSizeExecStatus, execStatus == null ? 0
                            : execStatus.length());
                    maxSizeValue = Math.max(maxSizeValue, entry.getValue().toString().length());
                }
            }

            maxSizeItf = Math.max(maxSizeItf, HEADER_INTERFACE_NAME.length());
            maxSizeSvc = Math.max(maxSizeSvc, HEADER_SERVICE_NAME.length());
            maxSizeOp = Math.max(maxSizeOp, HEADER_OPERATION.length());
            maxSizeMep = Math.max(maxSizeMep, HEADER_MEP.length());
            maxSizeExecStatus = Math.max(maxSizeExecStatus, HEADER_EXEC_STATUS.length());
            maxSizeValue = Math.max(maxSizeValue, HEADER_VALUE.length());

            final String rowHeaderFormat = "| %-" + maxSizeItf + "s | %-" + maxSizeSvc + "s | %-"
                    + maxSizeOp + "s | %-"
                    + maxSizeMep + "s | %-" + maxSizeExecStatus + "s || %-" + maxSizeValue + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_INTERFACE_NAME,
                    HEADER_SERVICE_NAME, HEADER_OPERATION, HEADER_MEP, HEADER_EXEC_STATUS,
                    HEADER_VALUE);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            if (values != null) {
                final String rowFormat = "| %-" + maxSizeItf + "s | %-" + maxSizeSvc + "s | %-"
                        + maxSizeOp + "s | %-"
                        + maxSizeMep + "s | %-" + maxSizeExecStatus + "s || %" + maxSizeValue
                        + "d |";
                for (final Map.Entry<ServiceProviderInvocationKey, Long> entry : values.entrySet()) {

                    final ServiceProviderInvocationKey key = entry.getKey();
                    final String interfaceName = key.getInterfaceName();
                    final String serviceName = key.getServiceName();
                    final String operation = key.getOperation();
                    final String mep = key.getMep();
                    final String execStatus = key.getExecStatus().toString();

                    this.shell.getPrintStream().println(
                            String.format(rowFormat,
                                    interfaceName == null ? "" : interfaceName,
                                    serviceName == null ? "" : serviceName,
                                    operation == null ? "" : operation,
                                    mep == null ? "" : mep,
                                    execStatus == null ? "" : execStatus, entry.getValue().longValue()));
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final ComponentMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, String.format(
                    Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
        }
    }
}
