/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */

package org.ow2.petals.cli.extension.command.monitoring.mo.components.se.flowable;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.AbstractComponentMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.DatabaseConnectionPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.SeFlowableMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.exception.MonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.exception.MonitoringServiceErrorException;

/**
 * The monitoring sub-function about the database connection pool of the SE Flowable.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DatabaseConnectionPool extends AbstractComponentMonitoringSubFunction implements MonitoringSubFunction {

    private static final String SUBFCT_NAME = "database-connection-pool";

    private static final String SUBFCT_DESCRIPTION = "Database connection pool statistics";

    /**
     * Header of the metric value associated to the max number of active connections of the connection pool
     */
    public static final String HEADER_ACTIVE_CONNECTIONS_MAX = "ActiveConnectionsMax";

    /**
     * Header of the metric value associated to the current number of active connections of the connection pool
     */
    public static final String HEADER_ACTIVE_CONNECTIONS_CURRENT = "ActiveConnectionsCurrent";

    /**
     * Header of the metric value associated to the max number of idle connections of the connection pool
     */
    public static final String HEADER_IDLE_CONNECTIONS_MAX = "IdleConnectionsMax";

    /**
     * Header of the metric value associated to the current number of idle connections of the connection pool
     */
    public static final String HEADER_IDLE_CONNECTIONS_CURRENT = "IdleConnectionsCurrent";

    /**
     * Header of the metric value associated to the max active connection size of the connection pool
     */
    public static final String HEADER_MAX_ACTIVE_SIZE = "MaxActiveSize";

    /**
     * Header of the metric value associated to the max idle connection size of the connection pool
     */
    public static final String HEADER_MAX_IDLE_SIZE = "MaxIdleSize";

    @Override
    public String getName() {
        return SUBFCT_NAME;
    }

    @Override
    public String getDescription() {
        return SUBFCT_DESCRIPTION;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException, MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);
            final String componentIdentifier = cli.getOptionValue(COMPONENT_NAME_SHORT_OPTION);

            final JMXClient client = getJMXClient();
            try {
                final SeFlowableMonitoringServiceClient seFlowableClient = client
                        .getSeFlowableMonitoringServiceClient(componentIdentifier);
                final DatabaseConnectionPoolMetrics databaseConnectionPoolMetrics = seFlowableClient
                        .getDatabaseConnectionPoolMetrics();

                final StringBuilder reply = new StringBuilder();
                final Long maxActiveSize = databaseConnectionPoolMetrics.getMaxActiveSize();
                if (maxActiveSize != null) {
                    reply.append(String.format("%s:%d ", HEADER_MAX_ACTIVE_SIZE, maxActiveSize.longValue()));
                }

                final Long maxIdleSize = databaseConnectionPoolMetrics.getMaxIdleSize();
                if (maxIdleSize != null) {
                    reply.append(String.format("%s:%d ", HEADER_MAX_IDLE_SIZE, maxIdleSize.longValue()));
                }

                final Long activeConnectionsMax = databaseConnectionPoolMetrics.getActiveConnectionsMaxSize();
                if (activeConnectionsMax != null) {
                    reply.append(
                            String.format("%s:%d ", HEADER_ACTIVE_CONNECTIONS_MAX, activeConnectionsMax.longValue()));
                }

                final Long activeConnectionsCurrent = databaseConnectionPoolMetrics.getActiveConnectionsSize();
                if (activeConnectionsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_ACTIVE_CONNECTIONS_CURRENT,
                            activeConnectionsCurrent.longValue()));
                }

                final Long idleConnectionsMax = databaseConnectionPoolMetrics.getIdleConnectionsMaxSize();
                if (idleConnectionsMax != null) {
                    reply.append(String.format("%s:%d ", HEADER_IDLE_CONNECTIONS_MAX, idleConnectionsMax.longValue()));
                }

                final Long idleConnectionsCurrent = databaseConnectionPoolMetrics.getIdleConnectionsSize();
                if (idleConnectionsCurrent != null) {
                    reply.append(String.format("%s:%d ", HEADER_IDLE_CONNECTIONS_CURRENT,
                            idleConnectionsCurrent.longValue()));
                }
                this.shell.getPrintStream().println(reply.toString());

            } catch (final MonitoringServiceDoesNotExistException e) {
                throw new MonitoringSubFunctionException(this,
                        String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST, componentIdentifier), e);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final MonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

}
