/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.remote;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.container.transporter.remote.exception.TcpTransporterMonitoringServiceErrorException;

/**
 * The base class for the monitoring sub-functions about delivered messages by the TCP transporter of the Petals
 * container
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public abstract class AbstractDeliveredMessages extends AbstractMonitoringSubFunction implements MonitoringSubFunction {

    /**
     * The option to get the list of containers
     */
    public static final String LIST_CONTAINERS_LONG_OPTION = "list-containers";

    private static final Option LIST_CONTAINERS_OPTION = Option.builder().required(false).numberOfArgs(0)
            .desc("List the containers for which at least one message was delivered.")
            .longOpt(LIST_CONTAINERS_LONG_OPTION).build();

    /**
     * The option to query delivered messages of containers
     */
    public static final String QUERY_CONTAINERS_LONG_OPTION = "query-containers";

    private static final Option QUERY_CONTAINERS_OPTION = Option.builder().required(false).optionalArg(true)
            .numberOfArgs(2).desc("Query about containers").longOpt(QUERY_CONTAINERS_LONG_OPTION).build();

    /**
     * The option to get the complete table of values
     */
    public static final String TABLE_SHORT_OPTION = "t";

    private static final Option TABLE_OPTION = Option.builder(TABLE_SHORT_OPTION).required(false)
            .desc("Get the complete table of values").build();

    /**
     * Query attribute to get index name
     */
    public static final String QUERY_ATTR_NAME = "NAME";

    /**
     * Query attribute to get the number of delivered messages that have a delivery status 'PENDING', required the index
     * on which the query must be run
     */
    public static final String QUERY_ATTR_DELIVERY_STATUS_PENDING = "PENDING";

    /**
     * Query attribute to get the number of delivered messages that have a delivery status 'SUCCEEDED', required the
     * index on which the query must be run
     */
    public static final String QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED = "SUCCEEDED";

    /**
     * Query attribute to get the number of delivered messages that have a delivery status 'ERROR', required the index
     * on which the query must be run
     */
    public static final String QUERY_ATTR_DELIVERY_STATUS_ERROR = "ERROR";

    /**
     * Index of the field 'container' in the key of the metric 'outgoing delivered message'
     */
    // TODO: Move the definition of indexes in a library common with
    // petals-kernel
    private final short REMOTE_CONTAINER_IDX = 0;

    /**
     * Index of the field 'delivery status' in the key of the metric 'delivered outgoing message'
     */
    private final short DELIVERY_STATUS_IDX = 1;

    private final static String HEADER_REMOTE_CONTAINER = "Remote Container";

    private final static String HEADER_DELIVERY_STATUS = "Delivery Status";

    private final static String HEADER_VALUE = "Value";

    /**
     * The sub-function options
     */
    private Options options;

    /**
     * The shell in which the sub-function is running
     */
    private Shell shell;

    public AbstractDeliveredMessages() {
        super();
        this.createOptions();
    }

    // TODO: Options declaration must be reviewed to match:
    // [--list-containers] |
    // [--query-containers [NAME] | [PENDING|SUCCEEDED|ERROR <index>]] |
    // [-t]
    private void createOptions() {
        this.options = new Options();
        this.options.addOption(LIST_CONTAINERS_OPTION);
        this.options.addOption(QUERY_CONTAINERS_OPTION);
        this.options.addOption(TABLE_OPTION);
    }

    @Override
    public Options getOptions() {
        return this.options;
    }

    @Override
    public void resetOptions() {
        this.createOptions();
    }

    @Override
    public void setShell(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public Shell getShell() {
        return this.shell;
    }

    @Override
    public void execute(final String[] args)
            throws MonitoringSubFunctionBadArgumentNumberException, MonitoringSubFunctionInvalidArgumentException,
            MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args, true);

            if (cli.hasOption(LIST_CONTAINERS_LONG_OPTION) && !cli.hasOption(QUERY_CONTAINERS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listContainers(false, null);

            } else if (!cli.hasOption(LIST_CONTAINERS_LONG_OPTION) && cli.hasOption(QUERY_CONTAINERS_LONG_OPTION)
                    && !cli.hasOption(TABLE_SHORT_OPTION)) {

                this.listContainers(true, cli.getOptionValues(QUERY_CONTAINERS_LONG_OPTION));

            } else if (!cli.hasOption(LIST_CONTAINERS_LONG_OPTION) && !cli.hasOption(QUERY_CONTAINERS_LONG_OPTION)
                    && cli.hasOption(TABLE_SHORT_OPTION)) {

                this.fullTable();

            } else {
                throw new MonitoringSubFunctionBadArgumentNumberException(this);
            }

        } catch (final UnrecognizedOptionException e) {
            throw new MonitoringSubFunctionBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new MonitoringSubFunctionMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new MonitoringSubFunctionMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new MonitoringSubFunctionInvalidException(this, e);
        }
    }

    protected abstract Map<String[], Long> getMetrics(final JMXClient client)
            throws TcpTransporterMonitoringServiceErrorException, TcpTransporterMonitoringServiceDoesNotExistException,
            ConnectionErrorException;

    /**
     * List containers for which a metric 'delivered messages' is available
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the metric value.
     */
    private void listContainers(final boolean hasQuery, final String[] queryArgs)
            throws MonitoringSubFunctionException {

        try {
            final Map<String[], Long> values = this.getMetrics(getJMXClient());
            if (hasQuery) {
                this.query(values, queryArgs, REMOTE_CONTAINER_IDX, QUERY_CONTAINERS_OPTION);
            } else {
                final List<String> containers = new ArrayList<>();
                for (final String[] key : values.keySet()) {
                    if (!containers.contains(key[REMOTE_CONTAINER_IDX])) {
                        containers.add(key[REMOTE_CONTAINER_IDX]);
                    }
                }

                for (final String container : containers) {
                    this.shell.getPrintStream().println(container);
                }
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }

    private void query(final Map<String[], Long> values, final String[] queryArgs, final short indexOfIndex,
            final Option queryOption) throws MonitoringSubFunctionException {

        if (queryArgs == null || queryArgs.length == 0 || queryArgs[0] == null || queryArgs[0].isEmpty()) {
            throw new MonitoringSubFunctionMissingArgumentException(this, queryOption);
        } else {
            // No other arguments is accepted has option value
            for (final String optionValue : queryArgs) {
                if (optionValue.startsWith("-")) {
                    // Another argument is given as option value
                    throw new MonitoringSubFunctionBadArgumentNumberException(this);
                }
            }

            if (QUERY_ATTR_NAME.equals(queryArgs[0])) {
                final List<String> containers = new ArrayList<>();
                for (final String[] key : values.keySet()) {
                    if (!containers.contains(key[REMOTE_CONTAINER_IDX])) {
                        containers.add(key[REMOTE_CONTAINER_IDX]);
                    }
                }

                for (final String container : containers) {
                    this.shell.getPrintStream().println(String.format("%s!%s", container, container));
                }
            } else {
                final Map<String, Long> valuesByDeliveryStatus;
                if (QUERY_ATTR_DELIVERY_STATUS_PENDING.equals(queryArgs[0])) {
                    valuesByDeliveryStatus = this.extractValuesForGivenDeliveryStatus(values, "PENDING");
                } else if (QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED.equals(queryArgs[0])) {
                    valuesByDeliveryStatus = this.extractValuesForGivenDeliveryStatus(values, "SUCCEEDED");
                } else if (QUERY_ATTR_DELIVERY_STATUS_ERROR.equals(queryArgs[0])) {
                    valuesByDeliveryStatus = this.extractValuesForGivenDeliveryStatus(values, "ERROR");
                } else {
                    throw new MonitoringSubFunctionInvalidArgumentException(this, queryOption, queryArgs[0]);
                }

                final String index = queryArgs.length == 2 ? queryArgs[1] : null;
                if (index != null) {
                    final Long value = valuesByDeliveryStatus.get(index);
                    if (value == null) {
                        this.shell.getPrintStream().println(String.format("%d", 0));
                    } else {
                        this.shell.getPrintStream().println(String.format("%s", value));
                    }
                } else {
                    for (final Map.Entry<String, Long> entry : valuesByDeliveryStatus.entrySet()) {
                        this.shell.getPrintStream().println(String.format("%s!%d", entry.getKey(), entry.getValue()));
                    }
                }
            }
        }
    }

    /**
     * Retrieve all values associated to the given delivery status from all values
     * 
     * @param values
     *            All values
     * @param deliveryStatus
     *            The given delivery status
     * @return Values limited to the given delivery status
     */
    private Map<String, Long> extractValuesForGivenDeliveryStatus(final Map<String[], Long> values, final String deliveryStatus) {
        final Map<String, Long> results = new HashMap<>();
        for (final Map.Entry<String[], Long> entry : values.entrySet()) {
            final String[] key = entry.getKey();
            if (key[DELIVERY_STATUS_IDX].equals(deliveryStatus)) {
                final Long agregatedValue = results.get(key[REMOTE_CONTAINER_IDX]);
                results.put(key[REMOTE_CONTAINER_IDX],
                        agregatedValue == null ? entry.getValue() : agregatedValue + entry.getValue());
            }
        }
        return results;
    }

    /**
     * Print the full table
     * 
     * @throws MonitoringSubFunctionException
     *             An error occurs getting the consolidated value.
     */
    private void fullTable() throws MonitoringSubFunctionException {

        try {
            final Map<String[], Long> values = this.getMetrics(getJMXClient());

            int maxSizeContainer = 0;
            int maxSizeDeliveryStatus = 0;
            int maxSizeValue = 0;
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {
                final String[] key = entry.getKey();
                maxSizeContainer = Math.max(maxSizeContainer,
                        key[REMOTE_CONTAINER_IDX] == null ? 0 : key[REMOTE_CONTAINER_IDX].length());
                maxSizeDeliveryStatus = Math.max(maxSizeDeliveryStatus,
                        key[DELIVERY_STATUS_IDX] == null ? 0 : key[DELIVERY_STATUS_IDX].length());
                maxSizeValue = Math.max(maxSizeValue, entry.getValue().toString().length());
            }

            maxSizeContainer = Math.max(maxSizeContainer, HEADER_REMOTE_CONTAINER.length());
            maxSizeDeliveryStatus = Math.max(maxSizeDeliveryStatus, HEADER_DELIVERY_STATUS.length());
            maxSizeValue = Math.max(maxSizeValue, HEADER_VALUE.length());

            final String rowHeaderFormat = "| %-" + maxSizeContainer + "s | %-" + maxSizeDeliveryStatus + "s || %-"
                    + maxSizeValue + "s |";
            final String header = String.format(rowHeaderFormat, HEADER_REMOTE_CONTAINER, HEADER_DELIVERY_STATUS,
                    HEADER_VALUE);
            this.shell.getPrintStream().println(header);
            for (int i = 0; i < header.length(); i++) {
                this.shell.getPrintStream().append('-');
            }
            this.shell.getPrintStream().append('\n');

            final String rowFormat = "| %-" + maxSizeContainer + "s | %-" + maxSizeDeliveryStatus + "s || %"
                    + maxSizeValue + "d |";
            for (final Map.Entry<String[], Long> entry : values.entrySet()) {

                final String[] key = entry.getKey();

                this.shell.getPrintStream()
                        .println(String.format(rowFormat,
                                key[REMOTE_CONTAINER_IDX] == null ? "" : key[REMOTE_CONTAINER_IDX],
                                key[DELIVERY_STATUS_IDX] == null ? "" : key[DELIVERY_STATUS_IDX],
                                entry.getValue().longValue()));
            }

        } catch (final ConnectionErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceErrorException e) {
            throw new MonitoringSubFunctionException(this, e);
        } catch (final TcpTransporterMonitoringServiceDoesNotExistException e) {
            throw new MonitoringSubFunctionException(this, e);
        }
    }
}
