/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INFORMATION_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.SERVICE_CONTRACTS_REQUEST_COUNT;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.UNKNOWN_REQUEST_COUNT;

import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsRequestExecutionStatus;
import org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock;

/**
 * Unit tests of {@link IncomingHttpRequestsCount}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingHttpRequestsCountTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new IncomingHttpRequestsCount();
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-"
                    + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option",
                    IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION, e.getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-r' is missing.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER, });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
            assertTrue("The missing option '"
                    + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION + "' is missing.",
                    missingOptions.contains(IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>error occurs getting incoming HTTP request counter value.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the information
     * HTTP request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the information HTTP request counter
     * is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     */
    @Test
    public void informationRequestCountNull() throws MonitoringSubFunctionException {

        INFORMATION_REQUEST_COUNT = null;
        UNKNOWN_REQUEST_COUNT = Long.valueOf(123);
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put("wsService", Long.valueOf(123));
        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey("wspath", "wsOperation",
                "clientIpAddress", WsRequestExecutionStatus.SUCCEEDED), Long.valueOf(123));

        this.subFunction.execute(new String[] {
                "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse(
                "the metric value associated to the information HTTP request counter is displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_INFORMATION));
        assertTrue(
                "the metric value associated to the unknown HTTP request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_UNKNOWN));
        assertTrue("the metric value associated to the HTTP ws-request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_WS));
        assertTrue("the metric value associated to the service contract request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_SERVICE_CONTRACTS));

    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the unknown
     * HTTP request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the unknown HTTP request counter is
     * not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     */
    @Test
    public void unknownRequestCountNull() throws MonitoringSubFunctionException {

        INFORMATION_REQUEST_COUNT = Long.valueOf(123);
        UNKNOWN_REQUEST_COUNT = null;
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put("wsService", Long.valueOf(123));
        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey("wspath", "wsOperation",
                "clientIpAddress", WsRequestExecutionStatus.SUCCEEDED), Long.valueOf(123));

        this.subFunction.execute(new String[] {
                "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "The metric value associated to the information HTTP request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_INFORMATION));
        assertFalse("the metric value associated to the unknown HTTP request counter is displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_UNKNOWN));
        assertTrue("the metric value associated to the HTTP ws-request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_WS));
        assertTrue("the metric value associated to the service contract request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_SERVICE_CONTRACTS));

    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the service contract request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the service contract request counter is not returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     */
    @Test
    public void serviceContractRequestCountNull() throws MonitoringSubFunctionException {

        INFORMATION_REQUEST_COUNT = Long.valueOf(123);
        UNKNOWN_REQUEST_COUNT = Long.valueOf(123);
        SERVICE_CONTRACTS_REQUEST_COUNT = null;
        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey("wspath", "wsOperation", "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED), Long.valueOf(123));

        this.subFunction.execute(new String[] { "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("The metric value associated to the information HTTP request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_INFORMATION));
        assertTrue("the metric value associated to the unknown HTTP request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_UNKNOWN));
        assertTrue("the metric value associated to the HTTP ws-request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_WS));
        assertFalse("the metric value associated to the service contract request counter is displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_SERVICE_CONTRACTS));

    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>a null value is returned for the metric associated to the HTTP
     * ws-request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric value associated to the HTTP ws-request counter is not
     * returned,</li>
     * <li>other values are correctly displayed.</li>
     * </ul>
     */
    @Test
    public void wsRequestCountNull() throws MonitoringSubFunctionException {

        INFORMATION_REQUEST_COUNT = Long.valueOf(123);
        UNKNOWN_REQUEST_COUNT = Long.valueOf(123);
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put("wsService", Long.valueOf(123));
        INCOMING_WS_REQUEST_COUNT = null;

        this.subFunction.execute(new String[] {
                "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue(
                "The metric value associated to the information HTTP request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_INFORMATION));
        assertTrue(
                "the metric value associated to the unknown HTTP request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_UNKNOWN));
        assertFalse("the metric value associated to the HTTP ws-request counter is displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_WS));
        assertTrue("the metric value associated to the service contract request counter is not displayed",
                output.contains(IncomingHttpRequestsCount.LABEL_SERVICE_CONTRACTS));

    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void noNullValue() throws MonitoringSubFunctionException {

        final long informationRequestCount = 123;
        final long unknownRequestCount = 456;
        final long incomingWsRequestCount1 = 789;
        final long serviceContractsRequestCount1 = 342;
        final long serviceContractsRequestCount2 = 793;
        final long incomingWsRequestCount2 = 10;
        INFORMATION_REQUEST_COUNT = Long.valueOf(informationRequestCount);
        UNKNOWN_REQUEST_COUNT = Long.valueOf(unknownRequestCount);
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put("ws-service-1", Long.valueOf(serviceContractsRequestCount1));
        SERVICE_CONTRACTS_REQUEST_COUNT.put("ws-service-2", Long.valueOf(serviceContractsRequestCount2));
        INCOMING_WS_REQUEST_COUNT = new HashMap<>();
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey("wspath-1",
                "wsOperation", "clientIpAddress", WsRequestExecutionStatus.SUCCEEDED),
                incomingWsRequestCount1);
        INCOMING_WS_REQUEST_COUNT.put(new IncomingWsRequestKey("wspath-2",
                "wsOperation", "clientIpAddress", WsRequestExecutionStatus.SUCCEEDED),
                incomingWsRequestCount2);

        this.subFunction.execute(new String[] {
                "-" + IncomingHttpRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingHttpRequestsCount.REQUEST_NATURE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        boolean isMissingInformation = true;
        boolean isMissingUnknown = true;
        boolean isMissingWs = true;
        boolean isMissingServiceContracts = true;
        final StringTokenizer strTokenizer = new StringTokenizer(output);
        while (strTokenizer.hasMoreElements()) {
            final StringTokenizer metricTokenizer = new StringTokenizer(strTokenizer.nextToken(),
                    ":");
            final String metricLabel = metricTokenizer.nextToken();
            assertNotNull("missing metric label", metricLabel);
            final String metricValue = metricTokenizer.nextToken();
            assertNotNull("missing metric value", metricValue);
            if (IncomingHttpRequestsCount.LABEL_INFORMATION.equals(metricLabel)) {
                assertEquals("invalid value for information HTTP request count",
                        INFORMATION_REQUEST_COUNT, Long.valueOf(metricValue));
                isMissingInformation = false;
            } else if (IncomingHttpRequestsCount.LABEL_UNKNOWN.equals(metricLabel)) {
                assertEquals("invalid value for information HTTP request count",
                        UNKNOWN_REQUEST_COUNT, Long.valueOf(metricValue));
                isMissingUnknown = false;
            } else if (IncomingHttpRequestsCount.LABEL_WS.equals(metricLabel)) {
                isMissingWs = false;
                assertEquals("invalid value for information HTTP request count",
                        incomingWsRequestCount1 + incomingWsRequestCount2, Long
                                .valueOf(metricValue).longValue());

            } else if (IncomingHttpRequestsCount.LABEL_SERVICE_CONTRACTS.equals(metricLabel)) {
                isMissingServiceContracts = false;
                assertEquals("invalid value for information HTTP request count",
                        serviceContractsRequestCount1 + serviceContractsRequestCount2,
                        Long.valueOf(metricValue).longValue());

            } else {
                fail("Unexpected metric");
            }
        }
        assertFalse("information HTTP request count is missing", isMissingInformation);
        assertFalse("unknown HTTP request count is missing", isMissingUnknown);
        assertFalse("WS-request count is missing", isMissingWs);
        assertFalse("Service contract count is missing", isMissingServiceContracts);
    }
}
