/**
 * Copyright (c) 2015-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.SERVICE_CONTRACTS_REQUEST_COUNT;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock;

/**
 * Unit tests of {@link IncomingServiceContractRequestsCount}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingServiceContractRequestsCountTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new IncomingServiceContractRequestsCount();
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option", IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-services' is provided with an argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-services' is provided without argument,</li>
     * <li>the option '--query-services' is provided with a valid argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION,
                            "--" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                            IncomingServiceContractRequestsCount.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-services' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION,
                            "-" + IncomingServiceContractRequestsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-services' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION, e.getOption().getLongOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-services' is provided with an invalid argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {

        final String invalidArg = "invalid-arg";
        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION, invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION, e.getOption().getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-services' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                            IncomingServiceContractRequestsCount.QUERY_ATTR_NAME,
                            "-" + IncomingServiceContractRequestsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {

        try {
            this.subFunction
                    .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                            BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "-" + IncomingServiceContractRequestsCount.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--list-services' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                        "-" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--query-services' is provided with a valid argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                        "--" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                        IncomingServiceContractRequestsCount.QUERY_ATTR_NAME });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                        "-" + IncomingServiceContractRequestsCount.TABLE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-services' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException {

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                        "-" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-services' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException {

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                        "--" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                        IncomingServiceContractRequestsCount.QUERY_ATTR_NAME });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException {

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                        "-" + IncomingServiceContractRequestsCount.TABLE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-services' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming ws-request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        SERVICE_CONTRACTS_REQUEST_COUNT = null;
        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed", output.isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-services' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the incoming web-service contract request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        SERVICE_CONTRACTS_REQUEST_COUNT = null;
        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                        IncomingServiceContractRequestsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed", output.isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the incoming ws-request counter,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException, IOException {

        SERVICE_CONTRACTS_REQUEST_COUNT = null;
        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-list-services' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException, IOException {

        final long incomingWsRequestCount1 = 789;
        final String incomingWsRequestWs1 = "ws-1";
        final long incomingWsRequestCount2 = 10;
        final String incomingWsRequestWs2 = "ws-2";
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs1, incomingWsRequestCount1);
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs2, incomingWsRequestCount2);

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.LIST_SERVICES_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbServices = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean serviceFound = false;
            for (final String webService : SERVICE_CONTRACTS_REQUEST_COUNT.keySet()) {
                if (line.equals(webService)) {
                    serviceFound = true;
                    nbServices++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", serviceFound);
        }
        assertEquals("unexpected operation number", SERVICE_CONTRACTS_REQUEST_COUNT.size(), nbServices);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-query-services' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException, IOException {

        final long incomingWsRequestCount1 = 789;
        final String incomingWsRequestWs1 = "ws-1";
        final long incomingWsRequestCount2 = 10;
        final String incomingWsRequestWs2 = "ws-2";
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs1, incomingWsRequestCount1);
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs2, incomingWsRequestCount2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWs1, Long.valueOf(incomingWsRequestCount1));
        expectedResults.put(incomingWsRequestWs2, Long.valueOf(incomingWsRequestCount2));

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                        IncomingServiceContractRequestsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbServices = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String serviceNameReturned = tokenizer.nextToken();
            assertFalse("No service name displayed", serviceNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean serviceFound = false;
            for (final String service : SERVICE_CONTRACTS_REQUEST_COUNT.keySet()) {
                if (serviceNameReturned.equals(service)) {
                    serviceFound = true;
                    assertEquals("Unexpected name value", serviceNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected service: [" + line + "]", serviceFound);
            nbServices++;
        }
        assertEquals("unexpected service number", expectedResults.size(), nbServices);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-services' is provided with the argument "COUNT",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_11() throws MonitoringSubFunctionException, IOException {

        final long incomingWsRequestCount1 = 789;
        final String incomingWsRequestWs1 = "ws-1";
        final long incomingWsRequestCount2 = 10;
        final String incomingWsRequestWs2 = "ws-2";
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs1, incomingWsRequestCount1);
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs2, incomingWsRequestCount2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWs1, Long.valueOf(incomingWsRequestCount1));
        expectedResults.put(incomingWsRequestWs2, Long.valueOf(incomingWsRequestCount2));

        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                        IncomingServiceContractRequestsCount.QUERY_ATTR_COUNT });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbServices = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String serviceNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", serviceNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean serviceFound = false;
            for (final String service : SERVICE_CONTRACTS_REQUEST_COUNT.keySet()) {
                if (serviceNameReturned.equals(service)) {
                    serviceFound = true;
                    assertEquals("Unexpected value", expectedResults.get(serviceNameReturned), Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected service: [" + line + "]", serviceFound);
            nbServices++;
        }
        assertEquals("unexpected service number", expectedResults.size(), nbServices);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-services' is provided with the arguments "COUNT" and an existing operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     */
    @Test
    public void testExecute_12() throws MonitoringSubFunctionException, IOException {

        final long incomingWsRequestCount1 = 789;
        final String incomingWsRequestWs1 = "ws-1";
        final long incomingWsRequestCount2 = 10;
        final String incomingWsRequestWs2 = "ws-2";
        SERVICE_CONTRACTS_REQUEST_COUNT = new HashMap<>();
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs1, incomingWsRequestCount1);
        SERVICE_CONTRACTS_REQUEST_COUNT.put(incomingWsRequestWs2, incomingWsRequestCount2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWs1, Long.valueOf(incomingWsRequestCount1));

        final String queriedService = incomingWsRequestWs1;
        this.subFunction
                .execute(new String[] { "-" + IncomingServiceContractRequestsCount.COMPONENT_NAME_SHORT_OPTION,
                        BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                        "-" + IncomingServiceContractRequestsCount.QUERY_SERVICES_LONG_OPTION,
                        IncomingServiceContractRequestsCount.QUERY_ATTR_COUNT, queriedService });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected value", expectedResults.get(queriedService), Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }
}
