/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.framework;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.apache.commons.cli.Options;
import org.junit.Test;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringObject;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.MonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Unit tests of {@link AbstractMonitoredComponent}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AbstractMonitoredComponentTest {

    /**
     * <p>
     * Check {@link AbstractMonitoredComponent#getSubFunctions()} against a
     * dedicated sub-function list set to <code>null</code>.
     * </p>
     * <p>
     * Expected result: only the sub-functions provided by the CDK are
     * available.
     * </p>
     */
    @Test
    public void testGetSubFunctions_NullDedicatedSubFunctions() {
        final MonitoringObject mo = new AbstractMonitoredComponent() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public List<MonitoringSubFunction> getDedicatedSubFunctions() {
                // No dedicated sub-functions
                return null;
            }
            
        };

        assertEquals("unexpected sub-functions",
                Arrays.asList((MonitoringSubFunction[]) ReflectionHelper.getPrivateFieldValue(
                        AbstractMonitoredComponent.class, null, "CDK_SUBFUNCTIONS")),
                mo.getSubFunctions());
    }

    /**
     * <p>
     * Check {@link AbstractMonitoredComponent#getSubFunctions()} against an
     * empty dedicated sub-function list.
     * </p>
     * <p>
     * Expected result: only the sub-functions provided by the CDK are
     * available.
     * </p>
     */
    @Test
    public void testGetSubFunctions_EmptyDedicatedSubFunctions() {
        final MonitoringObject mo = new AbstractMonitoredComponent() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public List<MonitoringSubFunction> getDedicatedSubFunctions() {
                // No dedicated sub-functions
                return Collections.emptyList();
            }

        };

        assertEquals("unexpected sub-functions",
                Arrays.asList((MonitoringSubFunction[]) ReflectionHelper.getPrivateFieldValue(
                        AbstractMonitoredComponent.class, null, "CDK_SUBFUNCTIONS")),
                mo.getSubFunctions());
    }

    /**
     * <p>
     * Check {@link AbstractMonitoredComponent#getSubFunctions()} against a
     * dedicated sub-function list set to a valid value.
     * </p>
     * <p>
     * Expected result: the sub-functions is set of both cdk sub-functions and
     * dedicated sub-functions.
     * </p>
     */
    @Test
    public void testGetSubFunctions_DedicatedSubFunctions() {
        final MonitoringSubFunction dedicatedSubFunction = new MonitoringSubFunction() {

            @Override
            public void setShell(final Shell shell) {
                // NOP
            }

            @Override
            public void resetOptions() {
                // NOP
            }

            @Override
            public Shell getShell() {
                return null;
            }

            @Override
            public Options getOptions() {
                return null;
            }

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public void execute(String[] args)
                    throws MonitoringSubFunctionBadArgumentNumberException,
                    MonitoringSubFunctionInvalidArgumentException,
                    MonitoringSubFunctionMissingArgumentException,
                    MonitoringSubFunctionMissingOptionsException,
                    MonitoringSubFunctionInvalidException, MonitoringSubFunctionException {
                // NOP
            }
        };

        final MonitoringObject mo = new AbstractMonitoredComponent() {

            @Override
            public String getName() {
                return null;
            }

            @Override
            public String getDescription() {
                return null;
            }

            @Override
            public List<MonitoringSubFunction> getDedicatedSubFunctions() {
                // No dedicated sub-functions
                return Arrays.asList(new MonitoringSubFunction[] { dedicatedSubFunction });
            }

        };

        final List<MonitoringSubFunction> expectedSubFunctions = new ArrayList<MonitoringSubFunction>();
        expectedSubFunctions.addAll(Arrays
                .asList((MonitoringSubFunction[]) ReflectionHelper.getPrivateFieldValue(
                        AbstractMonitoredComponent.class, null, "CDK_SUBFUNCTIONS")));
        expectedSubFunctions.add(dedicatedSubFunction);

        assertEquals("unexpected sub-functions", expectedSubFunctions, mo.getSubFunctions());
    }

}
