/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.se.flowable;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock.PROCESS_DEFINITIONS_INFORMATION;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable.SeFlowableMonitoringServiceClientMock;

/**
 * Unit tests of {@link ProcessDefinitionsCount}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ProcessDefinitionsCountTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new ProcessDefinitionsCount();
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void error_noOptionProvided() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void error_unrecognizedOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void error_componentOptionWithoutValue() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option", ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void error_onlyValidComponentOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-process-definitions' is provided with an argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void error_listProcessDefinitionsWithValue() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-process-definitions' is provided without argument,</li>
     * <li>the option '--query-process-definitions' is provided with a valid argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void error_listOptionWithQueryOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION,
                    "--" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                    ProcessDefinitionsCount.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-process-definitions' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void error_listOptionWithTableOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION,
                    "-" + ProcessDefinitionsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-process-definitions' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void error_queryOptionWithoutValue() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION, e.getOption().getLongOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-process-definitions' is provided with an invalid argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void error_queryOptionWithInvalidValue() throws MonitoringSubFunctionException {

        final String invalidArg = "invalid-arg";
        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION, invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION, e.getOption().getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-process-definitions' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void error_queryOptionWithTableOption() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                    ProcessDefinitionsCount.QUERY_ATTR_NAME, "-" + ProcessDefinitionsCount.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void error_tableOptionWithValue() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                    SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + ProcessDefinitionsCount.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist,</li>
     * <li>the option '--list-process-definitions' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the non existent component
     * </p>
     */
    @Test
    public void error_listOptionWithNonexistentComponentId() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist,</li>
     * <li>the option '--query-process_defintions' is provided with a valid argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the non existent component
     * </p>
     */
    @Test
    public void error_queryOptionWithNonexistentComponentId() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_NAME });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the non existent component
     * </p>
     */
    @Test
    public void error_tableOptionWithNonexistentComponentId() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + ProcessDefinitionsCount.TABLE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--list-process-definitions' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void error_errorListingProcessDefinitions() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void error_errorQueryingProcessDefinitions() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_NAME });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void error_errorOnTableOption() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + ProcessDefinitionsCount.TABLE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--list-process-definitions' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the information of process definitions.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void nominal_listOptionWithNullValueAsProcessDefinitionInfo() throws MonitoringSubFunctionException {

        PROCESS_DEFINITIONS_INFORMATION = null;
        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the information of process definitions is displayed",
                output.isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the information of process definitions.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionWithNullValueAsProcessDefinitionInfo() throws MonitoringSubFunctionException {

        PROCESS_DEFINITIONS_INFORMATION = null;
        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the information of process definitions is displayed",
                output.isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the information of process definitions.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void nominal_tableOptionWithNullValueAsProcessDefinitionInfo()
            throws MonitoringSubFunctionException, IOException {

        PROCESS_DEFINITIONS_INFORMATION = null;
        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--list-process-definitions' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void nominal_listOption() throws MonitoringSubFunctionException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final String processDefinitionId2 = "process-definition-2";
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1, new Long[] { 0l, 0l, 0l, 0l });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2, new Long[] { 0l, 0l, 0l, 0l });

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.LIST_PROCESS_DEFINITIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbProcessDefinitions = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean processDefinitionFound = false;
            for (final Entry<String, Long[]> entry : PROCESS_DEFINITIONS_INFORMATION.entrySet()) {
                if (line.equals(entry.getKey())) {
                    processDefinitionFound = true;
                    nbProcessDefinitions++;
                }
            }
            assertTrue("Unexpected process definition: [" + line + "]", processDefinitionFound);
        }
        assertEquals("unexpected process definitions number", PROCESS_DEFINITIONS_INFORMATION.size(),
                nbProcessDefinitions);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForName() throws MonitoringSubFunctionException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 1;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long[]> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId1,
                new Long[] { activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        expectedResults.put(processDefinitionId2,
                new Long[] { activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbProcessDefinitions = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String processDefinitionIdReturned = tokenizer.nextToken();
            assertFalse("No process definition id displayed", processDefinitionIdReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean processDefinitionFound = false;
            for (final Entry<String, Long[]> entry : PROCESS_DEFINITIONS_INFORMATION.entrySet()) {
                if (processDefinitionIdReturned.equals(entry.getKey())) {
                    processDefinitionFound = true;
                    assertEquals("Unexpected process definition value", processDefinitionIdReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected process definition: [" + line + "]", processDefinitionFound);
            nbProcessDefinitions++;
        }
        assertEquals("unexpected number of process definitions", expectedResults.size(), nbProcessDefinitions);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the argument "ACTIVE",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForActiveCounterWithoutIndex()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 12;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId1, Long.valueOf(activeProcessInstances1));
        expectedResults.put(processDefinitionId2, Long.valueOf(activeProcessInstances2));

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_ACTIVE });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiProcessDefinitionsCounters(output, PROCESS_DEFINITIONS_INFORMATION, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the arguments "ACTIVE" and an existing process
     * definition,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForActiveCounterWithIndex() throws MonitoringSubFunctionException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 1;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId1, Long.valueOf(activeProcessInstances1));

        final String queriedProcessDefinition = processDefinitionId1;
        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_ACTIVE, queriedProcessDefinition });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected active counter value", expectedResults.get(queriedProcessDefinition),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbValues);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the argument "SUSPENDED",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForSuspendedCounterWithoutIndex()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 1;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId1, Long.valueOf(suspendedProcessInstances1));
        expectedResults.put(processDefinitionId2, Long.valueOf(suspendedProcessInstances2));

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_SUSPENDED });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiProcessDefinitionsCounters(output, PROCESS_DEFINITIONS_INFORMATION, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the arguments "SUSPENDED" and an existing
     * operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForSuspendedCounterWithIndex() throws MonitoringSubFunctionException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 1;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId1, Long.valueOf(suspendedProcessInstances1));

        final String queriedProcessDefinition = processDefinitionId1;
        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_SUSPENDED, queriedProcessDefinition });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected suspended counter value", expectedResults.get(queriedProcessDefinition),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected number of process definitions", expectedResults.size(), nbValues);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the argument "ENDED",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForEndedCounterWithoutIndex()
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 1;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId1, Long.valueOf(endedProcessInstances1));
        expectedResults.put(processDefinitionId2, Long.valueOf(endedProcessInstances2));

        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_ENDED });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiProcessDefinitionsCounters(output, PROCESS_DEFINITIONS_INFORMATION, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the SE Flowable identifier provided exists,</li>
     * <li>the option '--query-process-definitions' is provided with the arguments "ERROR" and an existing operation,
     * </li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>only result for the operation set is displayed,</li>
     * <li>only the value is displayed, not the index name.</li>
     * </ul>
     */
    @Test
    public void nominal_queryOptionForEndedCounterWithIndex() throws MonitoringSubFunctionException, IOException {

        final String processDefinitionId1 = "process-definition-1";
        final long activeProcessInstances1 = 1;
        final long suspendedProcessInstances1 = 3;
        final long endedProcessInstances1 = 5;
        final String processDefinitionId2 = "process-definition-2";
        final long activeProcessInstances2 = 78;
        final long suspendedProcessInstances2 = 90;
        final long endedProcessInstances2 = 37;
        PROCESS_DEFINITIONS_INFORMATION = new HashMap<>();
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId1,
                new Long[] { 0l, activeProcessInstances1, suspendedProcessInstances1, endedProcessInstances1 });
        PROCESS_DEFINITIONS_INFORMATION.put(processDefinitionId2,
                new Long[] { 0l, activeProcessInstances2, suspendedProcessInstances2, endedProcessInstances2 });

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(processDefinitionId2, Long.valueOf(endedProcessInstances2));

        final String queriedProcessDefinition = processDefinitionId2;
        this.subFunction.execute(new String[] { "-" + ProcessDefinitionsCount.COMPONENT_NAME_SHORT_OPTION,
                SeFlowableMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + ProcessDefinitionsCount.QUERY_PROCESS_DEFINITIONS_LONG_OPTION,
                ProcessDefinitionsCount.QUERY_ATTR_ENDED, queriedProcessDefinition });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbValues = 0;
        String value;
        while ((value = brOutput.readLine()) != null) {
            assertEquals("Unexpected ended counter value", expectedResults.get(queriedProcessDefinition),
                    Long.valueOf(value));
            nbValues++;
        }
        assertEquals("unexpected number of process definitions", expectedResults.size(), nbValues);
    }

    /**
     * 
     * @param output
     *            The output stream as {@link String}
     * @param dataSet
     *            Unit test data set
     * @param expectedResults
     *            expected results
     * @throws MonitoringSubFunctionException
     * @throws ConnectionErrorException
     * @throws IOException
     */
    private void assertCactiProcessDefinitionsCounters(final String output, final Map<String, Long[]> dataSet,
            final Map<String, Long> expectedResults)
            throws MonitoringSubFunctionException, ConnectionErrorException, IOException {

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String processDefinitionIdReturned = tokenizer.nextToken();
            assertFalse("No process definition id displayed", processDefinitionIdReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean processDefinitionFound = false;
            for (final Entry<String, Long[]> entry : dataSet.entrySet()) {
                if (processDefinitionIdReturned.equals(entry.getKey())) {
                    processDefinitionFound = true;
                    assertEquals("Unexpected counter value", expectedResults.get(processDefinitionIdReturned),
                            Long.valueOf(value));
                    break;
                }
            }
            assertTrue("Unexpected process definition: [" + line + "]", processDefinitionFound);
            nbOperations++;
        }
        assertEquals("unexpected number of process definitions", expectedResults.size(), nbOperations);
    }
}
