/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.remote;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.AbstractMonitoringSubFunctionTest;
import org.ow2.petals.jmx.api.mock.junit.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClientMock;

/**
 * Unit tests of {@link DeliveredIncomingMessages}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeliveredIncomingMessagesTest extends AbstractMonitoringSubFunctionTest {

    private final static TcpTransporterMonitoringServiceClientMock TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK = new TcpTransporterMonitoringServiceClientMock();

    private final static String CONTAINER_1 = "container-1";

    private final static int VALUE_PENDING_1 = 51;

    private final static int VALUE_SUCCEEDED_1 = 510;

    private final static int VALUE_ERROR_1 = 54;

    private final static String CONTAINER_2 = "container-2";

    private final static int VALUE_PENDING_2 = 53;

    private final static int VALUE_SUCCEEDED_2 = 535;

    private final static int VALUE_ERROR_2 = 57;

    static {
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addDeliveredIncomingMessagesMetrics(CONTAINER_1,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING, VALUE_PENDING_1);
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addDeliveredIncomingMessagesMetrics(CONTAINER_1,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED, VALUE_SUCCEEDED_1);
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addDeliveredIncomingMessagesMetrics(CONTAINER_1,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR, VALUE_ERROR_1);

        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addDeliveredIncomingMessagesMetrics(CONTAINER_2,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING, VALUE_PENDING_2);
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addDeliveredIncomingMessagesMetrics(CONTAINER_2,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED, VALUE_SUCCEEDED_2);
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addDeliveredIncomingMessagesMetrics(CONTAINER_2,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR, VALUE_ERROR_2);
    }

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new DeliveredIncomingMessages();
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void noArgumentProvided() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>an unrecognized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedArgument() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option listing containers is set,</li>
     * <li>any messages was already sent by the transporter.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void listingContainersWhenNoMessagesWasSent() throws MonitoringSubFunctionException {

        // Set the test environment
        final TcpTransporterMonitoringServiceClientMock tcpTransporterMonitoringServiceClientMock = new TcpTransporterMonitoringServiceClientMock();
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(tcpTransporterMonitoringServiceClientMock);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.LIST_CONTAINERS_LONG_OPTION });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertTrue("Output is empty", shell.getOutput().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option listing containers is set,</li>
     * <li>some messages was already sent by the transporter.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void listingContainersWhenMessagesWasSent() throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.LIST_CONTAINERS_LONG_OPTION });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(new String[] { CONTAINER_1, CONTAINER_2 }));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set but without arguments</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void queryingContainersWithoutValue() throws MonitoringSubFunctionException {

        // Set the test environment
        final TcpTransporterMonitoringServiceClientMock tcpTransporterMonitoringServiceClientMock = new TcpTransporterMonitoringServiceClientMock();
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(tcpTransporterMonitoringServiceClientMock);

        // Execute test
        try {
            subFunction
                    .execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option raises the error", DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                    e.getOption().getLongOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set but with an invalid value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void queryingContainersWithInvalidValue() throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        final String invalidValue = "invalid-value";
        try {
            subFunction.execute(
                    new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION, invalidValue });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("Unexpected option raises the error", DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                    e.getOption().getLongOpt());
            assertEquals("Unexpected invalid value raises the error", invalidValue, e.getValue());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get index name.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersByName() throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_NAME });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell,
                Arrays.asList(new String[] { CONTAINER_1 + "!" + CONTAINER_1, CONTAINER_2 + "!" + CONTAINER_2 }));
    }

    private static void assertQueryResult(final StringStreamShell shell, final List<String> expectedResults)
            throws IOException {
        // First, duplicate the list to avoid problem because of element removing
        final List<String> resultsToCheck = new ArrayList<>(expectedResults);

        final BufferedReader outputReader = new BufferedReader(new StringReader(shell.getOutput()));
        String line;
        while ((line = outputReader.readLine()) != null) {
            if (!resultsToCheck.remove(line.trim())) {
                fail("Unexpected line: " + line);
            }
        }
        if (resultsToCheck.size() > 0) {
            fail("All expected lines not in result");
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get index name,</li>
     * <li>any messages was already sent by the transporter.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, empty output returned.
     * </p>
     */
    @Test
    public void queryingContainersByNameWhenNoMessageWasSent() throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(new TcpTransporterMonitoringServiceClientMock());

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_NAME });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertTrue("Output is not empty", shell.getOutput().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get index name,</li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersByNameWithUnrecognizedArg() throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        try {
            subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                    DeliveredIncomingMessages.QUERY_ATTR_NAME, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get number of delivered message with message exchange status
     * 'Pending' for all remote containers.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetPendingMessagesForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(new String[] { CONTAINER_1 + "!" + Integer.toString(VALUE_PENDING_1),
                CONTAINER_2 + "!" + Integer.toString(VALUE_PENDING_2) }));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status
     * 'Succeeded' for all remote containers.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetSucceededMessagesForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(new String[] { CONTAINER_1 + "!" + Integer.toString(VALUE_SUCCEEDED_1),
                CONTAINER_2 + "!" + Integer.toString(VALUE_SUCCEEDED_2) }));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status 'Error'
     * for all remote containers.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetErrorMessagesForAllContainers()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(new String[] { CONTAINER_1 + "!" + Integer.toString(VALUE_ERROR_1),
                CONTAINER_2 + "!" + Integer.toString(VALUE_ERROR_2) }));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status
     * 'Pending' for all remote containers.</li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetPendingMessagesForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        try {
            subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                    DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status
     * 'Succeeded' for all remote containers.</li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetSucceededMessagesForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        try {
            subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                    DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status 'Error'
     * for all remote containers.</li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetErrorMessagesForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        try {
            subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                    DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get number of delivered message with message exchange status
     * 'Pending' for one remote container,</li>
     * <li>some messages were already sent to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetPendingMessagesForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING, CONTAINER_1 });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(Integer.toString(VALUE_PENDING_1)));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status
     * 'Succeeded' for one remote container,</li>
     * <li>some messages were already sent to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetSucceededMessagesForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED, CONTAINER_1 });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(Integer.toString(VALUE_SUCCEEDED_1)));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status 'Error'
     * for one remote container,</li>
     * <li>some messages were already sent to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetErrorMessagesForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR, CONTAINER_1 });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(Integer.toString(VALUE_ERROR_1)));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get number of delivered message with message exchange status
     * 'Pending' for one remote container,</li>
     * <li>any message was already sent to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetPendingMessagesForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(new TcpTransporterMonitoringServiceClientMock());

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING, CONTAINER_1 });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList("0"));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status
     * 'Succeeded' for one remote container,</li>
     * <li>any message was already sent to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetSucceededMessagesForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(new TcpTransporterMonitoringServiceClientMock());

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED, CONTAINER_1 });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList("0"));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of delivered message with message exchange status 'Error'
     * for one remote container,</li>
     * <li>any message was already sent to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetErrorMessagesForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(new TcpTransporterMonitoringServiceClientMock());

        // Execute test
        subFunction.execute(new String[] { "--" + DeliveredIncomingMessages.QUERY_CONTAINERS_LONG_OPTION,
                DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR, CONTAINER_1 });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList("0"));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option getting the full value table is given.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, all values are returned.
     * </p>
     */
    @Test
    public void table()
            throws MonitoringSubFunctionInvalidArgumentException, MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "-" + DeliveredIncomingMessages.TABLE_SHORT_OPTION });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        final BufferedReader outputReader = new BufferedReader(new StringReader(shell.getOutput()));
        String line;
        int lineNb = 0;
        while ((line = outputReader.readLine()) != null) {
            if (lineNb <= 1) {
                // NOP: we skip header lines
            } else {
                final StringTokenizer tokenizer = new StringTokenizer(line, "|");
                assertEquals("Unexpected number of values", 3, tokenizer.countTokens());
                final String containerName = tokenizer.nextToken().trim();
                if (CONTAINER_1.equals(containerName)) {
                    assertContainerValues(tokenizer, VALUE_PENDING_1, VALUE_SUCCEEDED_1, VALUE_ERROR_1);
                } else if (CONTAINER_2.equals(containerName)) {
                    assertContainerValues(tokenizer, VALUE_PENDING_2, VALUE_SUCCEEDED_2, VALUE_ERROR_2);
                } else {
                    fail("Unexpected container name: " + containerName);
                }
            }
            lineNb++;
        }
    }

    private static final void assertContainerValues(final StringTokenizer tokenizer, final int expectedPending,
            final int expectedSucceeded, final int expectedError) {

        final String deliveryStatus = tokenizer.nextToken().trim();
        if (DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_PENDING.equals(deliveryStatus)) {
            assertEquals("unexpected counter value", Integer.toString(expectedPending), tokenizer.nextToken().trim());
        } else if (DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_SUCCEEDED.equals(deliveryStatus)) {
            assertEquals("unexpected counter value", Integer.toString(expectedSucceeded), tokenizer.nextToken().trim());
        } else if (DeliveredIncomingMessages.QUERY_ATTR_DELIVERY_STATUS_ERROR.equals(deliveryStatus)) {
            assertEquals("unexpected counter value", Integer.toString(expectedError), tokenizer.nextToken().trim());
        } else {
            fail("Unexpected delivery status: " + deliveryStatus);
        }
    }
}
