/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.remote;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.container.transporter.AbstractMonitoringSubFunctionTest;
import org.ow2.petals.jmx.api.mock.junit.monitoring.container.transporter.remote.IncomingConnectionObjectPoolMetricsMock;
import org.ow2.petals.jmx.api.mock.junit.monitoring.container.transporter.remote.TcpTransporterMonitoringServiceClientMock;

/**
 * Unit tests of {@link IncomingConnections}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingConnectionsTest extends AbstractMonitoringSubFunctionTest {

    private final static TcpTransporterMonitoringServiceClientMock TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK = new TcpTransporterMonitoringServiceClientMock();

    private final static String CONTAINER_1 = "container-1";

    private final static int VALUE_ACTIVE_MAX_1 = 51;

    private final static int VALUE_ACTIVE_1 = 11;

    private final static int VALUE_IDLE_MAX_1 = 5;

    private final static int VALUE_IDLE_1 = 3;

    private final static int VALUE_ESTABLISHED_1 = 653;

    private final static String CONTAINER_2 = "container-2";

    private final static int VALUE_ACTIVE_MAX_2 = 25;

    private final static int VALUE_ACTIVE_2 = 21;

    private final static int VALUE_IDLE_MAX_2 = 2;

    private final static int VALUE_IDLE_2 = 1;

    private final static int VALUE_ESTABLISHED_2 = 356;

    static {
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addIncomingConnectionObjectPoolMetrics(CONTAINER_1,
                new IncomingConnectionObjectPoolMetricsMock(VALUE_ACTIVE_1, VALUE_ACTIVE_MAX_1, VALUE_IDLE_1,
                        VALUE_IDLE_MAX_1, VALUE_ESTABLISHED_1));
        TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK.addIncomingConnectionObjectPoolMetrics(CONTAINER_2,
                new IncomingConnectionObjectPoolMetricsMock(VALUE_ACTIVE_2, VALUE_ACTIVE_MAX_2, VALUE_IDLE_2,
                        VALUE_IDLE_MAX_2, VALUE_ESTABLISHED_2));
    }

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new IncomingConnections();
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void noArgumentProvided() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedArgument() throws MonitoringSubFunctionException {

        try {
            subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option listing containers is set,</li>
     * <li>any connection was already established by the transporter.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void listingContainersWhenNoConnectionWasEstablished() throws MonitoringSubFunctionException {

        // Set the test environment
        final TcpTransporterMonitoringServiceClientMock tcpTransporterMonitoringServiceClientMock = new TcpTransporterMonitoringServiceClientMock();
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(tcpTransporterMonitoringServiceClientMock);

        // Execute test
        subFunction.execute(new String[] { "--" + IncomingConnections.LIST_CONTAINERS_LONG_OPTION });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertTrue("Output is empty", shell.getOutput().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option listing containers is set,</li>
     * <li>some connections were already established by the transporter.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void listingContainersWhenConnectionsWereEstablished() throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + IncomingConnections.LIST_CONTAINERS_LONG_OPTION });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(new String[] { CONTAINER_1, CONTAINER_2 }));
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set but without arguments</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void queryingContainersWithoutValue() throws MonitoringSubFunctionException {

        // Set the test environment
        final TcpTransporterMonitoringServiceClientMock tcpTransporterMonitoringServiceClientMock = new TcpTransporterMonitoringServiceClientMock();
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(tcpTransporterMonitoringServiceClientMock);

        // Execute test
        try {
            subFunction.execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option raises the error", IncomingConnections.QUERY_CONTAINERS_LONG_OPTION,
                    e.getOption().getLongOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set but with an invalid value</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void queryingContainersWithInvalidValue() throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        final String invalidValue = "invalid-value";
        try {
            subFunction
                    .execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION, invalidValue });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("Unexpected option raises the error", IncomingConnections.QUERY_CONTAINERS_LONG_OPTION,
                    e.getOption().getLongOpt());
            assertEquals("Unexpected invalid value raises the error", invalidValue, e.getValue());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get index name.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersByName() throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION,
                IncomingConnections.QUERY_ATTR_NAME });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell,
                Arrays.asList(new String[] { CONTAINER_1 + "!" + CONTAINER_1, CONTAINER_2 + "!" + CONTAINER_2 }));
    }

    private static void assertQueryResult(final StringStreamShell shell, final List<String> expectedResults)
            throws IOException {
        // First, duplicate the list to avoid problem because of element removing
        final List<String> resultsToCheck = new ArrayList<>(expectedResults);

        final BufferedReader outputReader = new BufferedReader(new StringReader(shell.getOutput()));
        String line;
        while ((line = outputReader.readLine()) != null) {
            if (!resultsToCheck.remove(line.trim())) {
                fail("Unexpected line: " + line);
            }
        }
        if (resultsToCheck.size() > 0) {
            fail("All expected lines not in result");
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get index name,</li>
     * <li>any connections was already established by the transporter.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, empty output returned.
     * </p>
     */
    @Test
    public void queryingContainersByNameWhenNoMessageWasSent() throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(new TcpTransporterMonitoringServiceClientMock());

        // Execute test
        subFunction.execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION,
                IncomingConnections.QUERY_ATTR_NAME });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertTrue("Output is not empty", shell.getOutput().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get index name,</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersByNameWithUnrecognizedArg() throws MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        try {
            subFunction.execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION,
                    IncomingConnections.QUERY_ATTR_NAME, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get the max number of active connections for all remote containers.
     * </li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxActiveConnectionsForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        this.assertQueryAttrMetricValue(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS_MAX, VALUE_ACTIVE_MAX_1,
                VALUE_ACTIVE_MAX_2);
    }

    private void assertQueryAttrMetricValue(final String queryAttr, final int value1, final int value2)
            throws MonitoringSubFunctionInvalidArgumentException, MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionBadArgumentNumberException,
            MonitoringSubFunctionInvalidException, MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction
                .execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION, queryAttr });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(new String[] { CONTAINER_1 + "!" + Integer.toString(value1),
                CONTAINER_2 + "!" + Integer.toString(value2) }));

    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get the current number of active connections for all remote
     * containers.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentActiveConnectionsForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        this.assertQueryAttrMetricValue(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS, VALUE_ACTIVE_1,
                VALUE_ACTIVE_2);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get the max number of idle connections for all remote containers.
     * </li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxIdleConnectionsForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        this.assertQueryAttrMetricValue(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS_MAX, VALUE_IDLE_MAX_1,
                VALUE_IDLE_MAX_2);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get the current number of idle connections for all remote containers.
     * </li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentIdleConnectionsForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        this.assertQueryAttrMetricValue(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS, VALUE_IDLE_1, VALUE_IDLE_2);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get the number of established connections for all remote containers.
     * </li>
     * </ul>
     * <p>
     * Expected results: no error occurs
     * </p>
     */
    @Test
    public void queryingContainersToGetEstablishedConnectionsForAllContainers() throws MonitoringSubFunctionException,
            IOException {

        this.assertQueryAttrMetricValue(IncomingConnections.QUERY_ATTR_ESTABLISHED_CONNECTIONS, VALUE_ESTABLISHED_1,
                VALUE_ESTABLISHED_2);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get max number of active connections for all remote containers.</li>
     * <li>another unregonized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxActiveConnectionsForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        this.assertQueryAttrMetricValueWithAnotherUnrecognizedArg(
                IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS_MAX);
    }

    private void assertQueryAttrMetricValueWithAnotherUnrecognizedArg(final String queryAttr)
            throws MonitoringSubFunctionInvalidArgumentException, MonitoringSubFunctionMissingArgumentException,
            MonitoringSubFunctionMissingOptionsException, MonitoringSubFunctionInvalidException,
            MonitoringSubFunctionException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        try {
            subFunction
                    .execute(new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION, queryAttr, "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue("Error label is missing.", e.getMessage().startsWith(
                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get current number of active connections for all remote containers.
     * </li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentActiveConnectionsForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        this.assertQueryAttrMetricValueWithAnotherUnrecognizedArg(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get max number of idle connections for all remote containers.</li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxIdleConnectionsForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        this.assertQueryAttrMetricValueWithAnotherUnrecognizedArg(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS_MAX);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get current number of idle connections for all remote containers.
     * </li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentIdleConnectionsForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        this.assertQueryAttrMetricValueWithAnotherUnrecognizedArg(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying containers is set to get number of established connections for all remote containers.
     * </li>
     * <li>another unrecognized option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void queryingContainersToGetEstablishedConnectionsForAllContainersWithAnotherUnrecognizedArg()
            throws MonitoringSubFunctionException {

        this.assertQueryAttrMetricValueWithAnotherUnrecognizedArg(
                IncomingConnections.QUERY_ATTR_ESTABLISHED_CONNECTIONS);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get max number of active connections for one remote container,</li>
     * <li>some connection were already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxActiveConnectionsForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS_MAX,
                CONTAINER_1, VALUE_ACTIVE_MAX_1);
    }

    private void assertQueryAttrMetricValueForOneContainer(final String queryAttr, final String container,
            final int expectedValue) throws MonitoringSubFunctionInvalidArgumentException,
                    MonitoringSubFunctionMissingArgumentException, MonitoringSubFunctionMissingOptionsException,
                    MonitoringSubFunctionInvalidException, MonitoringSubFunctionException, IOException {

        // Set the test environment
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(TCP_TRANSPORTER_MONITORING_SERVICE_CLIENT_MOCK);

        // Execute test
        subFunction.execute(
                new String[] { "--" + IncomingConnections.QUERY_CONTAINERS_LONG_OPTION, queryAttr, container });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        assertQueryResult(shell, Arrays.asList(Integer.toString(expectedValue)));

    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get current number of active connections for one remote container,
     * </li>
     * <li>some connection were already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentActiveConnectionsForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS, CONTAINER_1,
                VALUE_ACTIVE_1);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get max number of idle connections for one remote container,</li>
     * <li>some connection were already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxIdleConnectionsForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS_MAX, CONTAINER_1,
                VALUE_IDLE_MAX_1);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get current number of idle connections for one remote container,</li>
     * <li>some connection were already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentIdleConnectionsForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS, CONTAINER_1,
                VALUE_IDLE_1);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get number of established connections for one remote container,</li>
     * <li>some connection were already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, the right value is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetEstablishedConnectionsForOneContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_ESTABLISHED_CONNECTIONS,
                CONTAINER_1, VALUE_ESTABLISHED_1);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get max number of active connections for one remote container,</li>
     * <li>any connection was already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxActiveConnectionsForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS_MAX,
                "unknown-container", 0);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get current number of active connections for one remote container,
     * </li>
     * <li>any connection was already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentActiveConnectionsForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_ACTIVE_CONNECTIONS,
                "unknown-container", 0);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get max number of idle connections for one remote container,</li>
     * <li>any connection was already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetMaxIdleConnectionsForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS_MAX,
                "unknown-container", 0);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get current number of idle connections for one remote container,</li>
     * <li>any connection was already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetCurrentIdleConnectionsForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_IDLE_CONNECTIONS,
                "unknown-container", 0);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option querying container is set to get number of established connections for one remote container,</li>
     * <li>any connection was already established to the remote container.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, value '0' is returned.
     * </p>
     */
    @Test
    public void queryingContainersToGetEstablishedConnectionsForOneUnknownContainer()
            throws MonitoringSubFunctionException, IOException {

        this.assertQueryAttrMetricValueForOneContainer(IncomingConnections.QUERY_ATTR_ESTABLISHED_CONNECTIONS,
                "unknown-container", 0);
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option getting the full value table is given.</li>
     * </ul>
     * <p>
     * Expected results: no error occurs, all values are returned.
     * </p>
     */
    @Test
    public void table() throws MonitoringSubFunctionException, IOException {

        // Set the test environment
        final String container1 = "container-1";
        final int container1_active = 2;
        final int container1_maxActive = 4;
        final int container1_idle = 1;
        final int container1_maxIdle = 3;
        final int container1_established = 101;
        final String container2 = "container-2";
        final int container2_active = 4;
        final int container2_maxActive = 8;
        final int container2_idle = 2;
        final int container2_maxIdle = 6;
        final int container2_established = 501;
        final TcpTransporterMonitoringServiceClientMock tcpTransporterMonitoringServiceClientMock = new TcpTransporterMonitoringServiceClientMock();
        tcpTransporterMonitoringServiceClientMock.addIncomingConnectionObjectPoolMetrics(container1,
                new IncomingConnectionObjectPoolMetricsMock(container1_active, container1_maxActive, container1_idle,
                        container1_maxIdle, container1_established));
        tcpTransporterMonitoringServiceClientMock.addIncomingConnectionObjectPoolMetrics(container2,
                new IncomingConnectionObjectPoolMetricsMock(container2_active, container2_maxActive, container2_idle,
                        container2_maxIdle, container2_established));
        this.jmxApi.registerTcpTransporterMonitoringServiceClient(tcpTransporterMonitoringServiceClientMock);

        // Execute test
        subFunction.execute(new String[] { "-" + IncomingConnections.TABLE_SHORT_OPTION });

        // Assertions
        assertTrue("Error output not empty", shell.getError().isEmpty());
        assertFalse("Output is empty", shell.getOutput().isEmpty());
        final BufferedReader outputReader = new BufferedReader(new StringReader(shell.getOutput()));
        String line;
        int lineNb = 0;
        while ((line = outputReader.readLine()) != null) {
            if (lineNb <= 1) {
                // NOP: we skip header lines
            } else {
                final StringTokenizer tokenizer = new StringTokenizer(line, "|");
                assertEquals("Unexpected number of values", 6, tokenizer.countTokens());
                final String containerName = tokenizer.nextToken().trim();
                if (container1.equals(containerName)) {
                    assertEquals("unexpected active", Integer.toString(container1_active),
                            tokenizer.nextToken().trim());
                    assertEquals("unexpected maxActive", Integer.toString(container1_maxActive),
                            tokenizer.nextToken().trim());
                    assertEquals("unexpected idle", Integer.toString(container1_idle), tokenizer.nextToken().trim());
                    assertEquals("unexpected maxIdle", Integer.toString(container1_maxIdle),
                            tokenizer.nextToken().trim());
                    assertEquals("unexpected established", Integer.toString(container1_established),
                            tokenizer.nextToken().trim());
                } else if (container2.equals(containerName)) {
                    assertEquals("unexpected active", Integer.toString(container2_active),
                            tokenizer.nextToken().trim());
                    assertEquals("unexpected maxActive", Integer.toString(container2_maxActive),
                            tokenizer.nextToken().trim());
                    assertEquals("unexpected idle", Integer.toString(container2_idle), tokenizer.nextToken().trim());
                    assertEquals("unexpected maxIdle", Integer.toString(container2_maxIdle),
                            tokenizer.nextToken().trim());
                    assertEquals("unexpected established", Integer.toString(container2_established),
                            tokenizer.nextToken().trim());
                } else {
                    fail("Unexpected container name: " + containerName);
                }
            }
            lineNb++;
        }
    }
}
