/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.mo.components.bc.soap;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_RESPONSE_TIMES;
import static org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock.INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.junit.Test;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.AbstractMonitoringSubFunction;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionBadArgumentNumberException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionInvalidArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingArgumentException;
import org.ow2.petals.cli.extension.command.monitoring.mo.api.exception.MonitoringSubFunctionMissingOptionsException;
import org.ow2.petals.cli.extension.command.monitoring.mo.components.framework.Constants;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.IncomingWsRequestKey;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.WsRequestExecutionStatus;
import org.ow2.petals.jmx.api.mock.monitoring.component.bc.soap.BcSoapMonitoringServiceClientMock;

/**
 * Unit tests of {@link IncomingWsRequestsResponseTimes}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class IncomingWsRequestsResponseTimesTest extends AbstractTestCase {

    @Override
    protected AbstractMonitoringSubFunction buildSubFunction() {
        return new IncomingWsRequestsResponseTimes();
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_00() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] {});
            fail(MonitoringSubFunctionMissingOptionsException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingOptionsException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingOptionsException.MISSING_OPTIONS));

            final List missingOptions = e.getMissingOptions();
            assertEquals("More than one missing options", 1, missingOptions.size());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testExecute_Error_01() throws MonitoringSubFunctionException {

        try {
            this.subFunction.execute(new String[] { "-z" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided without argument</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_02() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] { "-"
                    + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("Unexpected option",
                    IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    e.getOption().getOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>no other option is provided.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_03() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided with an argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_04() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                            "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                            "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_05() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "--" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    IncomingWsRequestsResponseTimes.QUERY_ATTR_NAME });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_06() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION,
                    "-" + IncomingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the missing argument.
     * </p>
     */
    @Test
    public void testExecute_Error_07() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION });
            fail(MonitoringSubFunctionMissingArgumentException.class.getName() + " is not thrown.");
        } catch (final MonitoringSubFunctionMissingArgumentException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            MonitoringSubFunctionMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option having a missing argument is not the expected one",
                    IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with an invalid argument.
     * </li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the invalid argument.
     * </p>
     */
    @Test
    public void testExecute_Error_08() throws MonitoringSubFunctionException {


        final String invalidArg = "invalid-arg";
        try {
            this.subFunction.execute(new String[] {
                            "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                            "--" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                            invalidArg });
            fail(MonitoringSubFunctionInvalidArgumentException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionInvalidArgumentException e) {
            // Expected exception
            assertEquals("The option having an invalid argument is not the expected one",
                    IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION, e.getOption()
                            .getLongOpt());
            assertEquals("The invalid value is not the expected one", invalidArg, e.getValue());
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_09() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "--" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                    IncomingWsRequestsResponseTimes.QUERY_ATTR_NAME,
                    "-" + IncomingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against argument management, where:
     * </p>
     * <ul>
     * <li>the option '-n' is provided with an argument,</li>
     * <li>the option '-t' is provided with an argument.</li>
     * </ul>
     * <p>
     * Expected results: an error occurs about the bad number of arguments.
     * </p>
     */
    @Test
    public void testExecute_Error_10() throws MonitoringSubFunctionException {


        try {
            this.subFunction.execute(new String[] {
                    "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                    BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                    "-" + IncomingWsRequestsResponseTimes.TABLE_SHORT_OPTION, "dummy-arg" });
            fail(MonitoringSubFunctionBadArgumentNumberException.class.getName()
                    + " is not thrown.");
        } catch (final MonitoringSubFunctionBadArgumentNumberException e) {
            // Expected exception
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    MonitoringSubFunctionBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--list-operations' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_00() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '--query-operations' is provided with a valid argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_01() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "--" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided does not exist,</li>
     * <li>the option '-t' is provided without argument.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs about the unexisting component
     * </p>
     */
    @Test
    public void testExecute_02() throws MonitoringSubFunctionException {

        this.expectedEx.expect(MonitoringSubFunctionException.class);
        this.expectedEx.expectMessage(String.format(Constants.ERROR_MSG_COMP_DOES_NOT_EXIST,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT));

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT,
                "-" + IncomingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_03() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_04() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "--" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_NAME });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>an error occurs getting the operation list from the probe.</li>
     * </ul>
     * <p>
     * Expected results: An error occurs
     * </p>
     */
    @Test(expected = MonitoringSubFunctionException.class)
    public void testExecute_05() throws MonitoringSubFunctionException {

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR,
                "-" + IncomingWsRequestsResponseTimes.TABLE_SHORT_OPTION });
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--list-operations' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_06() throws MonitoringSubFunctionException {

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with a valid argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_07() throws MonitoringSubFunctionException {

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("the metric value associated to the incoming ws-request counter is displayed",
                output.isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-t' is provided without argument,</li>
     * <li>a null value is returned for the metric associated to the outgoing
     * ws-request response times,</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>no operation is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_08() throws MonitoringSubFunctionException,
            IOException {

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.TABLE_SHORT_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        final String header = brOutput.readLine();
        assertTrue("No header", header != null && !header.isEmpty());
        final String sepLine = brOutput.readLine();
        assertTrue("No line", sepLine != null && !sepLine.isEmpty());
        final String end = brOutput.readLine();
        assertNull("Extra character displayed: [" + end + "]", end);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-list-operations' is provided without argument,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_09() throws MonitoringSubFunctionException,
            IOException {

        final Long[] incomingWsRequestResponseTimes1 = new Long[] { Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789) };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long[] incomingWsRequestResponseTimes2 = new Long[] { Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10) };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1,
                incomingWsRequestWsOperation1, "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimes1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2,
                incomingWsRequestWsOperation2, "clientIpAddress",
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimes2);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.LIST_OPERATIONS_LONG_OPTION });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            boolean operationFound = false;
            for (final Map.Entry<IncomingWsRequestKey, Long[]> entry : INCOMING_WS_REQUEST_RESPONSE_TIMES
                    .entrySet()) {
                final IncomingWsRequestKey key = entry.getKey();
                if (line.equals(key.getWsOperation() + "@" + key.getWsPath())) {
                    operationFound = true;
                    nbOperations++;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
        }
        assertEquals("unexpected operation number", INCOMING_WS_REQUEST_RESPONSE_TIMES.size(),
                nbOperations);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '-query-operations' is provided with the argument "NAME",</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_10() throws MonitoringSubFunctionException,
            IOException {

        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] { Long.valueOf(789),
                Long.valueOf(789), Long.valueOf(789), Long.valueOf(789), Long.valueOf(789),
                Long.valueOf(789) };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] { Long.valueOf(10),
                Long.valueOf(10), Long.valueOf(10), Long.valueOf(10), Long.valueOf(10),
                Long.valueOf(10) };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] { Long.valueOf(56),
                Long.valueOf(56), Long.valueOf(56), Long.valueOf(56), Long.valueOf(56),
                Long.valueOf(56) };
        final Long[] incomingWsRequestResponseTimesFault3 = new Long[] { Long.valueOf(34),
                Long.valueOf(34), Long.valueOf(34), Long.valueOf(34), Long.valueOf(34),
                Long.valueOf(34) };
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] { Long.valueOf(9),
                Long.valueOf(9), Long.valueOf(9), Long.valueOf(9), Long.valueOf(9),
                Long.valueOf(9) };
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] { Long.valueOf(7),
                Long.valueOf(7), Long.valueOf(7), Long.valueOf(7), Long.valueOf(7),
                Long.valueOf(7) };

        INCOMING_WS_REQUEST_RESPONSE_TIMES = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault3);
        INCOMING_WS_REQUEST_RESPONSE_TIMES.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1, null);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2, null);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_NAME });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        final BufferedReader brOutput = new BufferedReader(new StringReader(output));
        int nbOperations = 0;
        String line;
        while ((line = brOutput.readLine()) != null) {
            final StringTokenizer tokenizer = new StringTokenizer(line,
                    org.ow2.petals.cli.extension.command.monitoring.mo.Constants.VALUE_SEPARATOR);
            final String operationNameReturned = tokenizer.nextToken();
            assertFalse("No operation name displayed", operationNameReturned.isEmpty());
            final String value = tokenizer.nextToken();
            assertFalse("No value displayed", value.isEmpty());

            boolean operationFound = false;
            for (final Map.Entry<IncomingWsRequestKey, Long[]> entry : INCOMING_WS_REQUEST_RESPONSE_TIMES
                    .entrySet()) {
                final IncomingWsRequestKey key = entry.getKey();
                if (operationNameReturned.equals(key.getWsOperation() + "@" + key.getWsPath())) {
                    operationFound = true;
                    assertEquals("Unexpected name value", operationNameReturned, value);
                    break;
                }
            }
            assertTrue("Unexpected operation: [" + line + "]", operationFound);
            nbOperations++;
        }
        assertEquals("unexpected operation number", expectedResults.size(), nbOperations);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededMin() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_min);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesSucceeded2_min);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_avg);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesSucceeded2_avg);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededMax() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_max);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesSucceeded2_max);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded10P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_10p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesSucceeded2_10p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded50P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_50p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesSucceeded2_50p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded90P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_90p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesSucceeded2_90p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultMin() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_min);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesFault2_min);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_avg);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesFault2_avg);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultMax() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_max);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesFault2_max);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Fault10P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_10p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesFault2_10p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute__Query_Fault50P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_50p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesFault2_50p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Fault90P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_90p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesFault2_90p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_ErrorMin() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_min);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesError2_min);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MIN });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_ErrorAvg() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_avg);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesError2_avg);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_AVG });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_ErrorMax() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_max);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesError2_max);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MAX });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error10P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_10p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesError2_10p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_10P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error50P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_50p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesError2_50p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_50P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and no operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error90P() throws MonitoringSubFunctionException,
            ConnectionErrorException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_90p);
        expectedResults.put(incomingWsRequestWsOperation2 + "@" + incomingWsRequestWsPath2,
                incomingWsRequestResponseTimesError2_90p);

        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_90P });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiIncomingResponseTimesOutput(output,
                INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS, expectedResults);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Valid_Operation()
            throws MonitoringSubFunctionException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_min);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_avg);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Valid_Operation()
            throws MonitoringSubFunctionException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_max);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_10p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_50p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "SUCCEEDED_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesSucceeded1_90p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_min);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_avg);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_max);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_10p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     */
    @Test
    public void testExecute_Query_Fault50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_50p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "FAULT_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     * @throws IOException
     * @throws NumberFormatException
     */
    @Test
    public void testExecute_Query_Fault90P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesFault1_90p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MIN" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     * @throws IOException
     * @throws NumberFormatException
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_min);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_AVG" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     * @throws IOException
     * @throws NumberFormatException
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_avg);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_MAX" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     * @throws IOException
     * @throws NumberFormatException
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_max);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_10P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     * @throws IOException
     * @throws NumberFormatException
     */
    @Test
    public void testExecute_Query_Error10P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_10p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument "ERROR_50P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     * 
     * @throws IOException
     * @throws NumberFormatException
     */
    @Test
    public void testExecute_Query_Error50P_With_Valid_Operation()
            throws MonitoringSubFunctionException, NumberFormatException, IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_50p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and a valid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>the metric values are returned and they have the right value.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error90P_With_Valid_Operation()
            throws MonitoringSubFunctionException,
            IOException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final Map<String, Long> expectedResults = new HashMap<>();
        expectedResults.put(incomingWsRequestWsOperation1 + "@" + incomingWsRequestWsPath1,
                incomingWsRequestResponseTimesError1_90p);

        final String queriedOperation = incomingWsRequestWsOperation1 + "@"
                + incomingWsRequestWsPath1;
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertFalse("No output", output.isEmpty());

        this.assertCactiResponseTimesOutputOfQueryWithArg(output, expectedResults,
                queriedOperation);
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_SucceededMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "SUCCEEDED_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Succeeded90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_SUCCEEDED_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_FaultMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Fault10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Fault50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "FAULT_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Fault90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_FAULT_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MIN" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_ErrorMin_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MIN, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_AVG" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_ErrorAvg_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_AVG, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_MAX" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_ErrorMax_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_MAX, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_10P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error10P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_10P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_50P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error50P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_50P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }

    /**
     * <p>
     * Test the method 'execute()' against processing, where:
     * </p>
     * <ul>
     * <li>a connection to a Petals ESB container is established,</li>
     * <li>the BC SOAP identifier provided exists,</li>
     * <li>the option '--query-operations' is provided with the argument
     * "ERROR_90P" and a invalid operation,</li>
     * <li>no null value is returned for a metric,</li>
     * <li>no error occurs.</li>
     * </ul>
     * <p>
     * Expected results:
     * </p>
     * <ul>
     * <li>no error,</li>
     * <li>an empty output is displayed.</li>
     * </ul>
     */
    @Test
    public void testExecute_Query_Error90P_With_Invalid_Operation()
            throws MonitoringSubFunctionException {

        final Long incomingWsRequestResponseTimesSucceeded1_min = Long.valueOf(2);
        final Long incomingWsRequestResponseTimesSucceeded1_avg = Long.valueOf(5);
        final Long incomingWsRequestResponseTimesSucceeded1_max = Long.valueOf(11);
        final Long incomingWsRequestResponseTimesSucceeded1_10p = Long.valueOf(3);
        final Long incomingWsRequestResponseTimesSucceeded1_50p = Long.valueOf(4);
        final Long incomingWsRequestResponseTimesSucceeded1_90p = Long.valueOf(7);
        final Long[] incomingWsRequestResponseTimesSucceeded1 = new Long[] {
                incomingWsRequestResponseTimesSucceeded1_max,
                incomingWsRequestResponseTimesSucceeded1_avg,
                incomingWsRequestResponseTimesSucceeded1_min,
                incomingWsRequestResponseTimesSucceeded1_10p,
                incomingWsRequestResponseTimesSucceeded1_50p,
                incomingWsRequestResponseTimesSucceeded1_90p };
        final String incomingWsRequestWsPath1 = "wspath-1";
        final String incomingWsRequestWsOperation1 = "wsOperation1";
        final Long incomingWsRequestResponseTimesSucceeded2_min = Long.valueOf(17);
        final Long incomingWsRequestResponseTimesSucceeded2_avg = Long.valueOf(20);
        final Long incomingWsRequestResponseTimesSucceeded2_max = Long.valueOf(26);
        final Long incomingWsRequestResponseTimesSucceeded2_10p = Long.valueOf(18);
        final Long incomingWsRequestResponseTimesSucceeded2_50p = Long.valueOf(19);
        final Long incomingWsRequestResponseTimesSucceeded2_90p = Long.valueOf(22);
        final Long[] incomingWsRequestResponseTimesSucceeded2 = new Long[] {
                incomingWsRequestResponseTimesSucceeded2_max,
                incomingWsRequestResponseTimesSucceeded2_avg,
                incomingWsRequestResponseTimesSucceeded2_min,
                incomingWsRequestResponseTimesSucceeded2_10p,
                incomingWsRequestResponseTimesSucceeded2_50p,
                incomingWsRequestResponseTimesSucceeded2_90p };
        final String incomingWsRequestWsPath2 = "wspath-2";
        final String incomingWsRequestWsOperation2 = "wsOperation2";
        final Long incomingWsRequestResponseTimesFault1_min = Long.valueOf(32);
        final Long incomingWsRequestResponseTimesFault1_avg = Long.valueOf(35);
        final Long incomingWsRequestResponseTimesFault1_max = Long.valueOf(41);
        final Long incomingWsRequestResponseTimesFault1_10p = Long.valueOf(33);
        final Long incomingWsRequestResponseTimesFault1_50p = Long.valueOf(34);
        final Long incomingWsRequestResponseTimesFault1_90p = Long.valueOf(39);
        final Long[] incomingWsRequestResponseTimesFault1 = new Long[] {
                incomingWsRequestResponseTimesFault1_max, incomingWsRequestResponseTimesFault1_avg,
                incomingWsRequestResponseTimesFault1_min, incomingWsRequestResponseTimesFault1_10p,
                incomingWsRequestResponseTimesFault1_50p, incomingWsRequestResponseTimesFault1_90p };
        final Long incomingWsRequestResponseTimesFault2_min = Long.valueOf(47);
        final Long incomingWsRequestResponseTimesFault2_avg = Long.valueOf(50);
        final Long incomingWsRequestResponseTimesFault2_max = Long.valueOf(56);
        final Long incomingWsRequestResponseTimesFault2_10p = Long.valueOf(48);
        final Long incomingWsRequestResponseTimesFault2_50p = Long.valueOf(49);
        final Long incomingWsRequestResponseTimesFault2_90p = Long.valueOf(54);
        final Long[] incomingWsRequestResponseTimesFault2 = new Long[] {
                incomingWsRequestResponseTimesFault2_max, incomingWsRequestResponseTimesFault2_avg,
                incomingWsRequestResponseTimesFault2_min, incomingWsRequestResponseTimesFault2_10p,
                incomingWsRequestResponseTimesFault2_50p, incomingWsRequestResponseTimesFault2_90p };
        final Long incomingWsRequestResponseTimesError1_min = Long.valueOf(62);
        final Long incomingWsRequestResponseTimesError1_avg = Long.valueOf(65);
        final Long incomingWsRequestResponseTimesError1_max = Long.valueOf(71);
        final Long incomingWsRequestResponseTimesError1_10p = Long.valueOf(63);
        final Long incomingWsRequestResponseTimesError1_50p = Long.valueOf(64);
        final Long incomingWsRequestResponseTimesError1_90p = Long.valueOf(69);
        final Long[] incomingWsRequestResponseTimesError1 = new Long[] {
                incomingWsRequestResponseTimesError1_max, incomingWsRequestResponseTimesError1_avg,
                incomingWsRequestResponseTimesError1_min, incomingWsRequestResponseTimesError1_10p,
                incomingWsRequestResponseTimesError1_50p, incomingWsRequestResponseTimesError1_90p };
        final Long incomingWsRequestResponseTimesError2_min = Long.valueOf(77);
        final Long incomingWsRequestResponseTimesError2_avg = Long.valueOf(80);
        final Long incomingWsRequestResponseTimesError2_max = Long.valueOf(86);
        final Long incomingWsRequestResponseTimesError2_10p = Long.valueOf(78);
        final Long incomingWsRequestResponseTimesError2_50p = Long.valueOf(79);
        final Long incomingWsRequestResponseTimesError2_90p = Long.valueOf(84);
        final Long[] incomingWsRequestResponseTimesError2 = new Long[] {
                incomingWsRequestResponseTimesError2_max, incomingWsRequestResponseTimesError2_avg,
                incomingWsRequestResponseTimesError2_min, incomingWsRequestResponseTimesError2_10p,
                incomingWsRequestResponseTimesError2_50p, incomingWsRequestResponseTimesError2_90p };

        // Incoming ws request response times grouped by service, operation and
        // exec status is required.
        INCOMING_WS_REQUEST_RESPONSE_TIMES = null;
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS = new HashMap<>();
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault1);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath1, incomingWsRequestWsOperation1, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError1);

        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.SUCCEEDED), incomingWsRequestResponseTimesSucceeded2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.FAULT), incomingWsRequestResponseTimesFault2);
        INCOMING_WS_REQUEST_RESPONSE_TIMES_GRP_BY_SVC_AND_STATUS.put(new IncomingWsRequestKey(
                incomingWsRequestWsPath2, incomingWsRequestWsOperation2, null,
                WsRequestExecutionStatus.ERROR), incomingWsRequestResponseTimesError2);

        final String queriedOperation = "unknown_operation";
        this.subFunction.execute(new String[] {
                "-" + IncomingWsRequestsResponseTimes.COMPONENT_NAME_SHORT_OPTION,
                BcSoapMonitoringServiceClientMock.DEFAULT_COMPONENT_IDENTIFIER,
                "-" + IncomingWsRequestsResponseTimes.QUERY_OPERATIONS_LONG_OPTION,
                IncomingWsRequestsResponseTimes.QUERY_ATTR_ERROR_90P, queriedOperation });

        final String error = shell.getError();
        final String output = shell.getOutput();
        assertTrue("An error occurs", error.isEmpty());
        assertTrue("No empty output", output.trim().isEmpty());
    }
}
